@@ -397,8 +397,8 @@ class CreateTestData(object):
             )
         model.Session.add(pr1)
         model.Session.add(pr2)
-        pkg1.resources.append(pr1)
-        pkg1.resources.append(pr2)
+        pkg1.resource_groups_all[0].resources_all.append(pr1)
+        pkg1.resource_groups_all[0].resources_all.append(pr2)
         pkg1.notes = u'''Some test notes
 
 ### A 3rd level heading
@@ -511,6 +511,7 @@ left arrow <
 
         model.repo.commit_and_remove()
 
+    # method used in DGU and all good tests elsewhere
     @classmethod
     def create_users(cls, user_dicts):
         needs_commit = False
@@ -523,7 +524,7 @@ left arrow <
 
     @classmethod
     def _create_user_without_commit(cls, name='', **user_dict):
-        if model.User.by_name(name) or (user_dict.get('openid') and model.User.by_openid(user_dict.get('openid'))):
+        if model.User.by_name(name) or (user_dict.get('open_id') and model.User.by_openid(user_dict.get('openid'))):
             log.warning('Cannot create user "%s" as it already exists.' % \
                             (name or user_dict['name']))
             return
@@ -537,6 +538,7 @@ left arrow <
         user = model.User(name=unicode(name), **user_dict)
         model.Session.add(user)
         cls.user_refs.append(user_ref)
+        return user
 
     @classmethod
     def create_user(cls, name='', **kwargs):
@@ -866,7 +868,6 @@ terms = ('A Novel By Tolstoy',
     'tolstoy',
     "Dave's books",
     "Roger's books",
-    'Other (Open)',
     'romantic novel',
     'book',
     '123',
@@ -887,7 +888,6 @@ german_translations = {
     'tolstoy': 'Tolstoi',
     "Dave's books": 'Daves Bucher',
     "Roger's books": 'Rogers Bucher',
-    'Other (Open)': 'Andere (Open)',
     'romantic novel': 'Liebesroman',
     'book': 'Buch',
     '456': 'Realismus',
