@@ -2,6 +2,7 @@ import datetime
 from pylons import config
 from sqlalchemy.sql import select
 import datetime
+import ckan.authz
 import ckan.model
 import ckan.misc
 import ckan.logic as logic
@@ -102,6 +103,10 @@ def resource_dictize(res, context):
         model = context['model']
         tracking = model.TrackingSummary.get_for_resource(res.url)
         resource['tracking_summary'] = tracking
+    # some urls do not have the protocol this adds http:// to these
+    url = resource['url']
+    if not (url.startswith('http://') or url.startswith('https://')):
+        resource['url'] = u'http://' + url
     return resource
 
 def related_dictize(rel, context):
@@ -305,6 +310,19 @@ def tag_list_dictize(tag_list, context):
             dictized = d.table_dictize(tag, context, capacity=capacity)
         else:
             dictized = d.table_dictize(tag, context)
+
+        # Add display_names to tag dicts. At first a tag's display_name is just
+        # the same as its name, but the display_name might get changed later
+        # (e.g.  translated into another language by the multilingual
+        # extension).
+        assert not dictized.has_key('display_name')
+        dictized['display_name'] = dictized['name']
+
+        if context.get('for_view'):
+            for item in plugins.PluginImplementations(
+                    plugins.ITagController):
+                dictized = item.before_view(dictized)
+
         result_list.append(dictized)
 
     return result_list
@@ -334,7 +352,6 @@ def user_list_dictize(obj_list, context,
 
     for obj in obj_list:
         user_dict = user_dictize(obj, context)
-        user_dict.pop('apikey')
         result_list.append(user_dict)
     return sorted(result_list, key=sort_key, reverse=reverse)
 
@@ -356,6 +373,25 @@ def user_dictize(user, context):
     result_dict['number_of_edits'] = user.number_of_edits()
     result_dict['number_administered_packages'] = user.number_administered_packages()
 
+    requester = context['user']
+
+    if not (ckan.authz.Authorizer().is_sysadmin(unicode(requester)) or
+            requester == user.name or
+            context.get('keep_sensitive_data', False)):
+        # If not sysadmin or the same user, strip sensible info
+        result_dict.pop('apikey', None)
+        result_dict.pop('reset_key', None)
+        result_dict.pop('email', None)
+
+    model = context['model']
+    session = model.Session
+
+    if context.get('with_related'):
+        related_items = session.query(model.Related).\
+                        filter(model.Related.owner_id==user.id).all()
+        result_dict['related_items'] = related_list_dictize(related_items,
+                                                            context)
+
     return result_dict
 
 def task_status_dictize(task_status, context):
@@ -507,3 +543,9 @@ def tag_to_api2(tag, context):
     # DEPRICIATED set api_version in context and use tag_to_api()
     context['api_version'] = 2
     return tag_to_api(tag, context)
+
+def user_following_user_dictize(follower, context):
+    return d.table_dictize(follower, context)
+
+def user_following_dataset_dictize(follower, context):
+    return d.table_dictize(follower, context)
