@@ -25,7 +25,6 @@ from pylons import config
 from routes import redirect_to as _redirect_to
 from routes import url_for as _routes_default_url_for
 from alphabet_paginate import AlphaPage
-from lxml.html import fromstring
 import i18n
 import ckan.exceptions
 from pylons import request
@@ -33,6 +32,8 @@ from pylons import session
 from pylons import c
 from pylons.i18n import _
 
+from lib.maintain import deprecated
+import ckan.model as model
 get_available_locales = i18n.get_available_locales
 get_locales_dict = i18n.get_locales_dict
 
@@ -153,9 +154,18 @@ def _add_i18n_to_url(url_to_amend, **kw):
     return url
 
 def lang():
-    ''' Reurn the language code for the current locale eg `en` '''
+    ''' Return the language code for the current locale eg `en` '''
     return request.environ.get('CKAN_LANG')
 
+def lang_native_name(lang=None):
+    ''' Return the langage name currently used in it's localised form
+        either from parameter or current environ setting'''
+    lang = lang or lang()
+    locale = get_locales_dict().get(lang)
+    if locale:
+        return locale.display_name or locale.english_name
+    return lang
+
 class Message(object):
     """A message returned by ``Flash.pop_messages()``.
 
@@ -367,46 +377,18 @@ def unselected_facet_items(facet, limit=10):
             facets.append(facet_item)
     return sorted(facets, key=lambda item: item['count'], reverse=True)[:limit]
 
-def facet_items(*args, **kwargs):
-    """
-    DEPRECATED: Use the new facet data structure, and `unselected_facet_items()`
-    """
-    _log.warning('Deprecated function: ckan.lib.helpers:facet_items().  Will be removed in v1.8')
-    # facet_items() used to need c passing as the first arg
-    # this is deprecated as pointless
-    # throws error if ckan.restrict_template_vars is True
-    # When we move to strict helpers then this should be removed as a wrapper
-    if len(args) > 2 or (len(args) > 0 and 'name' in kwargs) or (len(args) > 1 and 'limit' in kwargs):
-        if not asbool(config.get('ckan.restrict_template_vars', 'false')):
-            return _facet_items(*args[1:], **kwargs)
-        raise Exception('facet_items() calling has been changed. remove c in template %s or included one' % c.__template_name)
-    return _facet_items(*args, **kwargs)
-
-
-def _facet_items(name, limit=10):
-    if not c.facets or not c.facets.get(name):
-        return []
-    facets = []
-    for k, v in c.facets.get(name).items():
-        if not len(k.strip()):
-            continue
-        if not (name, k) in request.params.items():
-            facets.append((k, v))
-    return sorted(facets, key=lambda (k, v): v, reverse=True)[:limit]
-
 def facet_title(name):
     return config.get('search.facets.%s.title' % name, name.capitalize())
 
+@deprecated('Please use check_access instead.')
 def am_authorized(c, action, domain_object=None):
     ''' Deprecated. Please use check_access instead'''
     from ckan.authz import Authorizer
     if domain_object is None:
-        from ckan import model
         domain_object = model.System()
     return Authorizer.am_authorized(c, action, domain_object)
 
 def check_access(action, data_dict=None):
-    from ckan import model
     from ckan.logic import check_access as check_access_logic,NotAuthorized
 
     context = {'model': model,
@@ -421,7 +403,6 @@ def check_access(action, data_dict=None):
     return authorized
 
 def linked_user(user, maxlength=0):
-    from ckan import model
     if user in [model.PSEUDO_USER__LOGGED_IN, model.PSEUDO_USER__VISITOR]:
         return user
     if not isinstance(user, model.User):
@@ -439,7 +420,6 @@ def linked_user(user, maxlength=0):
                        url_for(controller='user', action='read', id=_name))
 
 def linked_authorization_group(authgroup, maxlength=0):
-    from ckan import model
     if not isinstance(authgroup, model.AuthorizationGroup):
         authgroup_name = unicode(authgroup)
         authgroup = model.AuthorizationGroup.get(authgroup_name)
@@ -453,7 +433,6 @@ def linked_authorization_group(authgroup, maxlength=0):
                        url_for(controller='authorization_group', action='read', id=displayname))
 
 def group_name_to_title(name):
-    from ckan import model
     group = model.Group.by_name(name)
     if group is not None:
         return group.display_name
@@ -462,8 +441,7 @@ def group_name_to_title(name):
 def markdown_extract(text, extract_length=190):
     if (text is None) or (text.strip() == ''):
         return ''
-    html = fromstring(markdown(text))
-    plain = html.xpath("string()")
+    plain = re.sub(r'<.*?>', '', markdown(text))
     return unicode(truncate(plain, length=extract_length, indicator='...', whole_word=True))
 
 def icon_url(name):
@@ -500,7 +478,7 @@ def format_icon(_format):
 
 def linked_gravatar(email_hash, size=100, default=None):
     return literal(
-        '<a href="https://gravatar.com/" target="_blank"' +
+        '<a href="https://gravatar.com/" target="_blank" ' +
         'title="%s">' % _('Update your avatar at gravatar.com') +
         '%s</a>' % gravatar(email_hash,size,default)
         )
@@ -582,8 +560,9 @@ def render_datetime(datetime_, date_format=None, with_hours=False):
     else:
         return ''
 
+@deprecated()
 def datetime_to_date_str(datetime_):
-    '''Takes a datetime.datetime object and returns a string of it
+    '''DEPRECATED: Takes a datetime.datetime object and returns a string of it
     in ISO format.
     '''
     return datetime_.isoformat()
@@ -625,7 +604,7 @@ def parse_rfc_2822_date(date_str, assume_utc=True):
 
     RFC 2822 is the date format used in HTTP headers.  It should contain timezone
     information, but that cannot be relied upon.
-    
+
     If date_str doesn't contain timezone information, then the 'assume_utc' flag
     determines whether we assume this string is local (with respect to the
     server running this code), or UTC.  In practice, what this means is that if
@@ -634,7 +613,7 @@ def parse_rfc_2822_date(date_str, assume_utc=True):
 
     If timezone information is available in date_str, then the returned datetime
     is 'aware', ie - it has an associated tz_info object.
-    
+
     Returns None if the string cannot be parsed as a valid datetime.
     """
     time_tuple = email.utils.parsedate_tz(date_str)
@@ -802,7 +781,6 @@ def convert_to_dict(object_type, objs):
                'message' : revision.message,}
         return rev
     import lib.dictization.model_dictize as md
-    import ckan.model as model
     converters = {'package' : md.package_dictize,
                   'revisions' : dictize_revision_list}
     converter = converters[object_type]
@@ -813,6 +791,85 @@ def convert_to_dict(object_type, objs):
         items.append(item)
     return items
 
+# these are the types of objects that can be followed
+_follow_objects = ['dataset', 'user']
+
+def follow_button(obj_type, obj_id):
+    '''Return a follow button for the given object type and id.
+
+    If the user is not logged in return an empty string instead.
+
+    :param obj_type: the type of the object to be followed when the follow
+        button is clicked, e.g. 'user' or 'dataset'
+    :type obj_type: string
+    :param obj_id: the id of the object to be followed when the follow button
+        is clicked
+    :type obj_id: string
+
+    :returns: a follow button as an HTML snippet
+    :rtype: string
+
+    '''
+    import ckan.logic as logic
+    obj_type = obj_type.lower()
+    assert obj_type in _follow_objects
+    # If the user is logged in show the follow/unfollow button
+    if c.user:
+        context = {'model' : model, 'session':model.Session, 'user':c.user}
+        action = 'am_following_%s' % obj_type
+        following = logic.get_action(action)(context, {'id': obj_id})
+        return snippet('snippets/follow_button.html',
+                       following=following,
+                       obj_id=obj_id,
+                       obj_type=obj_type)
+    return ''
+
+def follow_count(obj_type, obj_id):
+    '''Return the number of followers of an object.
+
+    :param obj_type: the type of the object, e.g. 'user' or 'dataset'
+    :type obj_type: string
+    :param obj_id: the id of the object
+    :type obj_id: string
+
+    :returns: the number of followers of the object
+    :rtype: int
+
+    '''
+    import ckan.logic as logic
+    obj_type = obj_type.lower()
+    assert obj_type in _follow_objects
+    action = '%s_follower_count' % obj_type
+    context = {'model' : model, 'session':model.Session, 'user':c.user}
+    return logic.get_action(action)(context, {'id': obj_id})
+
+def dashboard_activity_stream(user_id):
+    '''Return the dashboard activity stream of the given user.
+
+    :param user_id: the id of the user
+    :type user_id: string
+
+    :returns: an activity stream as an HTML snippet
+    :rtype: string
+
+    '''
+    import ckan.logic as logic
+    context = {'model' : model, 'session':model.Session, 'user':c.user}
+    return logic.get_action('dashboard_activity_list_html')(context, {'id': user_id})
+
+def get_request_param(parameter_name, default=None):
+    ''' This function allows templates to access query string parameters
+    from the request. This is useful for things like sort order in
+    searches. '''
+    return request.params.get(parameter_name, default)
+
+def render_markdown(data):
+    ''' returns the data as rendered markdown '''
+    # cope with data == None
+    if not data:
+        return ''
+    return literal(ckan.misc.MarkdownFormat().to_html(data))
+
 
 # these are the functions that will end up in `h` template helpers
 # if config option restrict_template_vars is true
@@ -832,7 +889,6 @@ __allowed_functions__ = [
            'subnav_link',
            'subnav_named_route',
            'default_group_type',
-           'facet_items',
            'facet_title',
          #  am_authorized, # deprecated
            'check_access',
@@ -865,8 +921,14 @@ __allowed_functions__ = [
            'snippet',
            'convert_to_dict',
            'activity_div',
+           'lang_native_name',
            'unselected_facet_items',
-    # imported into ckan.lib.helpers
+           'follow_button',
+           'follow_count',
+           'dashboard_activity_stream',
+           'get_request_param',
+           'render_markdown',
+           # imported into ckan.lib.helpers
            'literal',
            'link_to',
            'get_available_locales',
