@@ -2,6 +2,7 @@ import copy
 import formencode as fe
 import inspect
 from pylons.i18n import _
+from pylons import config
 
 class Missing(object):
     def __unicode__(self):
@@ -217,8 +218,22 @@ def validate(data, schema, context=None):
     context = context or {}
 
     assert isinstance(data, dict)
+
+    # store any empty lists in the data as they will get stripped out by
+    # the _validate function. We do this so we can differentiate between
+    # empty fields and missing fields when doing partial updates.
+    empty_lists = [key for key, value in data.items() if value == []]
+
     flattened = flatten_dict(data)
     converted_data, errors = _validate(flattened, schema, context)
+
+    # check config for partial update fix option
+    if config.get('ckan.fix_partial_updates', False):
+        # repopulate the empty lists
+        for key in empty_lists:
+            if key not in converted_data:
+                converted_data[(key,)] = []
+
     converted_data = unflatten(converted_data)
 
     errors_unflattened = unflatten(errors)
@@ -302,7 +317,6 @@ def _validate(data, schema, context):
             except StopOnError:
                 break
 
-
     return converted_data, errors
 
 
