@@ -1,21 +1,25 @@
 import logging
 from pylons import config, c
+from paste.deploy.converters import asbool
 
 from ckan import model
-from ckan.model import DomainObjectOperation
 from ckan.plugins import SingletonPlugin, implements, IDomainObjectModification
 from ckan.logic import get_action
+import ckan.model.domain_object as domain_object
 
 from common import (SearchIndexError, SearchError, SearchQueryError,
                     make_connection, is_available, SolrSettings)
 from index import PackageSearchIndex, NoopSearchIndex
-from query import TagSearchQuery, ResourceSearchQuery, PackageSearchQuery, QueryOptions, convert_legacy_parameters_to_solr
+from query import (TagSearchQuery, ResourceSearchQuery, PackageSearchQuery,
+                   QueryOptions, convert_legacy_parameters_to_solr)
 
 log = logging.getLogger(__name__)
 
 import sys
 import cgitb
 import warnings
+
+
 def text_traceback():
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
@@ -24,7 +28,7 @@ def text_traceback():
         ).strip()
     return res
 
-SIMPLE_SEARCH = config.get('ckan.simple_search', False)
+SIMPLE_SEARCH = asbool(config.get('ckan.simple_search', False))
 
 SUPPORTED_SCHEMA_VERSIONS = ['1.4']
 
@@ -37,7 +41,7 @@ DEFAULT_OPTIONS = {
     'ref_entity_with_attr': 'name',
     'all_fields': False,
     'search_tags': True,
-    'callback': None, # simply passed through
+    'callback': None,  # simply passed through
 }
 
 _INDICES = {
@@ -57,15 +61,18 @@ if SIMPLE_SEARCH:
     _INDICES['package'] = NoopSearchIndex
     _QUERIES['package'] = sql.PackageSearchQuery
 
+
 def _normalize_type(_type):
-    if isinstance(_type, model.DomainObject):
+    if isinstance(_type, domain_object.DomainObject):
         _type = _type.__class__
     if isinstance(_type, type):
         _type = _type.__name__
     return _type.strip().lower()
 
+
 def index_for(_type):
-    """ Get a SearchIndex instance sub-class suitable for the specified type. """
+    """ Get a SearchIndex instance sub-class suitable for
+        the specified type. """
     try:
         _type_n = _normalize_type(_type)
         return _INDICES[_type_n]()
@@ -73,23 +80,26 @@ def index_for(_type):
         log.warn("Unknown search type: %s" % _type)
         return NoopSearchIndex()
 
-def query_for( _type):
-    """ Get a SearchQuery instance sub-class suitable for the specified type. """
+
+def query_for(_type):
+    """ Get a SearchQuery instance sub-class suitable for the specified
+        type. """
     try:
         _type_n = _normalize_type(_type)
         return _QUERIES[_type_n]()
     except KeyError, ke:
         raise SearchError("Unknown search type: %s" % _type)
 
+
 def dispatch_by_operation(entity_type, entity, operation):
     """Call the appropriate index method for a given notification."""
     try:
         index = index_for(entity_type)
-        if operation == DomainObjectOperation.new:
+        if operation == domain_object.DomainObjectOperation.new:
             index.insert_dict(entity)
-        elif operation == DomainObjectOperation.changed:
+        elif operation == domain_object.DomainObjectOperation.changed:
             index.update_dict(entity)
-        elif operation == DomainObjectOperation.deleted:
+        elif operation == domain_object.DomainObjectOperation.deleted:
             index.remove_dict(entity)
         else:
             log.warn("Unknown operation: %s" % operation)
@@ -107,7 +117,7 @@ class SynchronousSearchPlugin(SingletonPlugin):
     def notify(self, entity, operation):
         if not isinstance(entity, model.Package):
             return
-        if operation != DomainObjectOperation.deleted:
+        if operation != domain_object.DomainObjectOperation.deleted:
             dispatch_by_operation(
                 entity.__class__.__name__,
                 get_action('package_show')(
@@ -115,60 +125,68 @@ class SynchronousSearchPlugin(SingletonPlugin):
                     {'id': entity.id}),
                 operation
             )
-        elif operation == DomainObjectOperation.deleted:
+        elif operation == domain_object.DomainObjectOperation.deleted:
             dispatch_by_operation(entity.__class__.__name__,
                                   {'id': entity.id}, operation)
         else:
             log.warn("Discarded Sync. indexing for: %s" % entity)
 
-def rebuild(package_id=None,only_missing=False,force=False,refresh=False):
+
+def rebuild(package_id=None, only_missing=False, force=False, refresh=False, defer_commit=False):
     '''
         Rebuilds the search index.
 
         If a dataset id is provided, only this dataset will be reindexed.
         When reindexing all datasets, if only_missing is True, only the
         datasets not already indexed will be processed. If force equals
-        True, if an execption is found, the exception will be logged, but
+        True, if an exception is found, the exception will be logged, but
         the process will carry on.
     '''
     from ckan import model
-    log.debug("Rebuilding search index...")
+    log.info("Rebuilding search index...")
 
     package_index = index_for(model.Package)
 
     if package_id:
         pkg_dict = get_action('package_show')(
-                    {'model': model, 'ignore_auth': True, 'validate': False},
-                    {'id': package_id})
+            {'model': model, 'ignore_auth': True, 'validate': False},
+            {'id': package_id})
+        log.info('Indexing just package %r...', pkg_dict['name'])
         package_index.remove_dict(pkg_dict)
         package_index.insert_dict(pkg_dict)
     else:
-        package_ids = [r[0] for r in model.Session.query(model.Package.id).filter(model.Package.state == 'active').all()]
+        package_ids = [r[0] for r in model.Session.query(model.Package.id).
+                       filter(model.Package.state == 'active').all()]
         if only_missing:
-            log.debug('Indexing only missing packages...')
+            log.info('Indexing only missing packages...')
             package_query = query_for(model.Package)
-            indexed_pkg_ids = set(package_query.get_all_entity_ids(max_results=len(package_ids)))
-            package_ids = set(package_ids) - indexed_pkg_ids     # Packages not indexed
+            indexed_pkg_ids = set(package_query.get_all_entity_ids(
+                max_results=len(package_ids)))
+            # Packages not indexed
+            package_ids = set(package_ids) - indexed_pkg_ids
 
             if len(package_ids) == 0:
-                log.debug('All datasets are already indexed')
+                log.info('All datasets are already indexed')
                 return
         else:
-            log.debug('Rebuilding the whole index...')
+            log.info('Rebuilding the whole index...')
             # When refreshing, the index is not previously cleared
             if not refresh:
                 package_index.clear()
 
         for pkg_id in package_ids:
             try:
-                package_index.insert_dict(
+                package_index.update_dict(
                     get_action('package_show')(
-                        {'model': model, 'ignore_auth': True, 'validate': False},
+                        {'model': model, 'ignore_auth': True,
+                         'validate': False},
                         {'id': pkg_id}
-                    )
+                    ),
+                    defer_commit
                 )
-            except Exception,e:
-                log.error('Error while indexing dataset %s: %s' % (pkg_id,str(e)))
+            except Exception, e:
+                log.error('Error while indexing dataset %s: %s' %
+                          (pkg_id, str(e)))
                 if force:
                     log.error(text_traceback())
                     continue
@@ -176,35 +194,46 @@ def rebuild(package_id=None,only_missing=False,force=False,refresh=False):
                     raise
 
     model.Session.commit()
-    log.debug('Finished rebuilding search index.')
+    log.info('Finished rebuilding search index.')
+
+
+def commit():
+    package_index = index_for(model.Package)
+    package_index.commit()
+    log.info('Commited pending changes on the search index')
 
 def check():
     from ckan import model
     package_query = query_for(model.Package)
 
     log.debug("Checking packages search index...")
-    pkgs_q = model.Session.query(model.Package).filter_by(state=model.State.ACTIVE)
+    pkgs_q = model.Session.query(model.Package).filter_by(
+        state=model.State.ACTIVE)
     pkgs = set([pkg.id for pkg in pkgs_q])
     indexed_pkgs = set(package_query.get_all_entity_ids(max_results=len(pkgs)))
     pkgs_not_indexed = pkgs - indexed_pkgs
-    print 'Packages not indexed = %i out of %i' % (len(pkgs_not_indexed), len(pkgs))
+    print 'Packages not indexed = %i out of %i' % (len(pkgs_not_indexed),
+                                                   len(pkgs))
     for pkg_id in pkgs_not_indexed:
         pkg = model.Session.query(model.Package).get(pkg_id)
         print pkg.revision.timestamp.strftime('%Y-%m-%d'), pkg.name
 
+
 def show(package_reference):
     from ckan import model
     package_query = query_for(model.Package)
 
     return package_query.get_index(package_reference)
 
+
 def clear(package_reference=None):
     from ckan import model
     package_index = index_for(model.Package)
     if package_reference:
-        log.debug("Clearing search index for dataset %s..." % package_reference)
-        package_index.delete_package({'id':package_reference})
-    else:
+        log.debug("Clearing search index for dataset %s..." %
+                  package_reference)
+        package_index.delete_package({'id': package_reference})
+    elif not SIMPLE_SEARCH:
         log.debug("Clearing search index...")
         package_index.clear()
 
@@ -252,24 +281,25 @@ def check_solr_schema_version(schema_file=None):
 
         url = solr_url.strip('/') + SOLR_SCHEMA_FILE_OFFSET
 
-        req = urllib2.Request(url = url)
+        req = urllib2.Request(url=url)
         if http_auth:
-            req.add_header('Authorization',http_auth)
+            req.add_header('Authorization', http_auth)
 
         res = urllib2.urlopen(req)
     else:
         url = 'file://%s' % schema_file
         res = urllib2.urlopen(url)
 
-    from lxml import etree
-    tree = etree.fromstring(res.read())
+    import xml.dom.minidom
+    tree = xml.dom.minidom.parseString(res.read())
 
-    version = tree.xpath('//schema/@version')
+    version = tree.documentElement.getAttribute('version')
     if not len(version):
-        raise SearchError('Could not extract version info from the SOLR schema, using file: \n%s' % url)
-    version = version[0]
+        raise SearchError('Could not extract version info from the SOLR'
+                          ' schema, using file: \n%s' % url)
 
     if not version in SUPPORTED_SCHEMA_VERSIONS:
-        raise SearchError('SOLR schema version not supported: %s. Supported versions are [%s]'
-                % (version,', '.join(SUPPORTED_SCHEMA_VERSIONS)))
+        raise SearchError('SOLR schema version not supported: %s. Supported'
+                          ' versions are [%s]'
+                          % (version, ', '.join(SUPPORTED_SCHEMA_VERSIONS)))
     return True
