@@ -1,7 +1,6 @@
 import socket
 import string
 import logging
-import itertools
 import collections
 import json
 
@@ -14,6 +13,7 @@ from ckan.model import PackageRelationship
 import ckan.model as model
 from ckan.plugins import (PluginImplementations,
                           IPackageController)
+import ckan.logic as logic
 
 log = logging.getLogger(__name__)
 
@@ -92,10 +92,10 @@ class PackageSearchIndex(SearchIndex):
     def remove_dict(self, pkg_dict):
         self.delete_package(pkg_dict)
 
-    def update_dict(self, pkg_dict):
-        self.index_package(pkg_dict)
+    def update_dict(self, pkg_dict, defer_commit=False):
+        self.index_package(pkg_dict, defer_commit)
 
-    def index_package(self, pkg_dict):
+    def index_package(self, pkg_dict, defer_commit=False):
         if pkg_dict is None:
             return
         pkg_dict['data_dict'] = json.dumps(pkg_dict)
@@ -122,10 +122,27 @@ class PackageSearchIndex(SearchIndex):
                 pkg_dict[key] = value
         pkg_dict.pop('extras', None)
 
-        #Add tags and groups
+        # add tags, removing vocab tags from 'tags' list and adding them as
+        # vocab_<tag name> so that they can be used in facets
+        non_vocab_tag_names = []
         tags = pkg_dict.pop('tags', [])
-        pkg_dict['tags'] = [tag['name'] for tag in tags]
+        context = {'model': model}
 
+        for tag in tags:
+            if tag.get('vocabulary_id'):
+                data = {'id': tag['vocabulary_id']}
+                vocab = logic.get_action('vocabulary_show')(context, data)
+                key = u'vocab_%s' % vocab['name']
+                if key in pkg_dict:
+                    pkg_dict[key].append(tag['name'])
+                else:
+                    pkg_dict[key] = [tag['name']]
+            else:
+                non_vocab_tag_names.append(tag['name'])
+
+        pkg_dict['tags'] = non_vocab_tag_names
+
+        # add groups
         groups = pkg_dict.pop('groups', [])
 
         # Capacity is different to the default only if using organizations
@@ -197,7 +214,6 @@ class PackageSearchIndex(SearchIndex):
         import hashlib
         pkg_dict['index_id'] = hashlib.md5('%s%s' % (pkg_dict['id'],config.get('ckan.site_id'))).hexdigest()
 
-
         for item in PluginImplementations(IPackageController):
             pkg_dict = item.before_index(pkg_dict)
 
@@ -206,7 +222,20 @@ class PackageSearchIndex(SearchIndex):
         # send to solr:
         try:
             conn = make_connection()
-            conn.add_many([pkg_dict])
+            commit = not defer_commit
+            conn.add_many([pkg_dict], _commit=commit)
+        except Exception, e:
+            log.exception(e)
+            raise SearchIndexError(e)
+        finally:
+            conn.close()
+
+        commit_debug_msg = 'Not commited yet' if defer_commit else 'Commited'
+        log.debug('Updated index for %s [%s]' % (pkg_dict.get('name'), commit_debug_msg))
+
+    def commit(self):
+        try:
+            conn = make_connection()
             conn.commit(wait_flush=False, wait_searcher=False)
         except Exception, e:
             log.exception(e)
@@ -214,7 +243,6 @@ class PackageSearchIndex(SearchIndex):
         finally:
             conn.close()
 
-        log.debug("Updated index for %s" % pkg_dict.get('name'))
 
     def delete_package(self, pkg_dict):
         conn = make_connection()
