@@ -4,6 +4,7 @@ import datetime
 from pylons.i18n import _
 from vdm.sqlalchemy.base import SQLAlchemySession
 
+import ckan.authz as authz
 import ckan.plugins as plugins
 import ckan.logic as logic
 import ckan.logic.schema
@@ -16,13 +17,15 @@ import ckan.lib.plugins as lib_plugins
 
 log = logging.getLogger(__name__)
 
-# define some shortcuts
-validate = ckan.lib.navl.dictization_functions.validate
-error_summary = logic.action.error_summary
-get_action = logic.get_action
-check_access = logic.check_access
+# Define some shortcuts
+# Ensure they are module-private so that they don't get loaded as available
+# actions in the action API.
+_validate = ckan.lib.navl.dictization_functions.validate
+_get_action = logic.get_action
+_check_access = logic.check_access
 NotFound = logic.NotFound
 ValidationError = logic.ValidationError
+_get_or_bust = logic.get_or_bust
 
 def _make_latest_rev_active(context, q):
 
@@ -56,14 +59,21 @@ def _make_latest_rev_active(context, q):
         context['latest_revision'] = latest_rev.revision_id
 
 def make_latest_pending_package_active(context, data_dict):
+    '''TODO: What does this function do?
 
+    You must be authorized to update the dataset.
+
+    :param id: the name or id of the dataset, e.g. ``'warandpeace'``
+    :type id: string
+
+    '''
     model = context['model']
     session = model.Session
     SQLAlchemySession.setattr(session, 'revisioning_disabled', True)
-    id = data_dict["id"]
+    id = _get_or_bust(data_dict, "id")
     pkg = model.Package.get(id)
 
-    check_access('make_latest_pending_package_active', context, data_dict)
+    _check_access('make_latest_pending_package_active', context, data_dict)
 
     #packages
     q = session.query(model.PackageRevision).filter_by(id=pkg.id)
@@ -99,9 +109,22 @@ def make_latest_pending_package_active(context, data_dict):
 
 
 def related_update(context, data_dict):
+    '''Update a related item.
+
+    You must be the owner of a related item to update it.
+
+    For further parameters see ``related_create()``.
+
+    :param id: the id of the related item to update
+    :type id: string
+
+    :returns: the updated related item
+    :rtype: dictionary
+
+    '''
     model = context['model']
     user = context['user']
-    id = data_dict["id"]
+    id = _get_or_bust(data_dict, "id")
 
     schema = context.get('schema') or ckan.logic.schema.default_related_schema()
     model.Session.remove()
@@ -111,14 +134,13 @@ def related_update(context, data_dict):
 
     if not related:
         logging.error('Could not find related ' + id)
-        raise NotFound(_('Related was not found.'))
-
-    check_access('related_update', context, data_dict)
-    data, errors = validate(data_dict, schema, context)
+        raise NotFound(_('Item was not found.'))
 
+    _check_access('related_update', context, data_dict)
+    data, errors = _validate(data_dict, schema, context)
     if errors:
         model.Session.rollback()
-        raise ValidationError(errors, error_summary(errors))
+        raise ValidationError(errors)
 
     related = model_save.related_dict_save(data, context)
     if not context.get('defer_commit'):
@@ -128,9 +150,23 @@ def related_update(context, data_dict):
 
 
 def resource_update(context, data_dict):
+    '''Update a resource.
+
+    To update a resource you must be authorized to update the dataset that the
+    resource belongs to.
+
+    For further parameters see ``resource_create()``.
+
+    :param id: the id of the resource to update
+    :type id: string
+
+    :returns: the updated resource
+    :rtype: string
+
+    '''
     model = context['model']
     user = context['user']
-    id = data_dict["id"]
+    id = _get_or_bust(data_dict, "id")
     schema = context.get('schema') or ckan.logic.schema.default_update_resource_schema()
     model.Session.remove()
 
@@ -141,13 +177,12 @@ def resource_update(context, data_dict):
         logging.error('Could not find resource ' + id)
         raise NotFound(_('Resource was not found.'))
 
-    check_access('resource_update', context, data_dict)
-
-    data, errors = validate(data_dict, schema, context)
+    _check_access('resource_update', context, data_dict)
 
+    data, errors = _validate(data_dict, schema, context)
     if errors:
         model.Session.rollback()
-        raise ValidationError(errors, error_summary(errors))
+        raise ValidationError(errors)
 
     rev = model.repo.new_revision()
     rev.author = user
@@ -164,7 +199,26 @@ def resource_update(context, data_dict):
 
 
 def package_update(context, data_dict):
+    '''Update a dataset (package).
 
+    You must be authorized to edit the dataset and the groups that it belongs
+    to.
+
+    Plugins may change the parameters of this function depending on the value
+    of the dataset's ``type`` attribute, see the ``IDatasetForm`` plugin
+    interface.
+
+    For further parameters see ``package_create()``.
+
+    :param id: the name or id of the dataset to update
+    :type id: string
+
+    :returns: the updated dataset (if 'return_package_dict' is True in the
+              context, which is the default. Otherwise returns just the
+              dataset id)
+    :rtype: dictionary
+
+    '''
     model = context['model']
     user = context['user']
     name_or_id = data_dict.get("id") or data_dict['name']
@@ -177,7 +231,7 @@ def package_update(context, data_dict):
     context["package"] = pkg
     data_dict["id"] = pkg.id
 
-    check_access('package_update', context, data_dict)
+    _check_access('package_update', context, data_dict)
 
     # get the schema
     package_plugin = lib_plugins.lookup_package_plugin(pkg.type)
@@ -196,11 +250,15 @@ def package_update(context, data_dict):
         except TypeError:
             package_plugin.check_data_dict(data_dict)
 
-    data, errors = validate(data_dict, schema, context)
+    data, errors = _validate(data_dict, schema, context)
+    log.debug('package_update validate_errs=%r user=%s package=%s data=%r',
+              errors, context.get('user'),
+              context.get('package').name if context.get('package') else '',
+              data)
 
     if errors:
         model.Session.rollback()
-        raise ValidationError(errors, error_summary(errors))
+        raise ValidationError(errors)
 
     rev = model.repo.new_revision()
     rev.author = user
@@ -215,13 +273,21 @@ def package_update(context, data_dict):
         item.edit(pkg)
     if not context.get('defer_commit'):
         model.repo.commit()
-    return get_action('package_show')(context, data_dict)
+
+    log.debug('Updated object %s' % str(pkg.name))
+
+    return_id_only = context.get('return_id_only', False)
+
+    output = data_dict['id'] if return_id_only \
+            else _get_action('package_show')(context, {'id': data_dict['id']})
+
+    return output
 
 def package_update_validate(context, data_dict):
     model = context['model']
     user = context['user']
 
-    id = data_dict["id"]
+    id = _get_or_bust(data_dict, "id")
     model.Session.remove()
     model.Session()._context = context
 
@@ -241,14 +307,12 @@ def package_update_validate(context, data_dict):
     except AttributeError:
         schema = package_plugin.form_to_db_schema()
 
-    check_access('package_update', context, data_dict)
-
-    data, errors = validate(data_dict, schema, context)
-
+    _check_access('package_update', context, data_dict)
 
+    data, errors = _validate(data_dict, schema, context)
     if errors:
         model.Session.rollback()
-        raise ValidationError(errors, error_summary(errors))
+        raise ValidationError(errors)
     return data
 
 
@@ -270,14 +334,33 @@ def _update_package_relationship(relationship, comment, context):
     return rel_dict
 
 def package_relationship_update(context, data_dict):
+    '''Update a relationship between two datasets (packages).
 
+    You must be authorized to edit both the subject and the object datasets.
+
+    :param id: the id of the package relationship to update
+    :type id: string
+    :param subject: the name or id of the dataset that is the subject of the
+        relationship (optional)
+    :type subject: string
+    :param object: the name or id of the dataset that is the object of the
+        relationship (optional)
+    :param type: the type of the relationship, one of ``'depends_on'``,
+        ``'dependency_of'``, ``'derives_from'``, ``'has_derivation'``,
+        ``'links_to'``, ``'linked_from'``, ``'child_of'`` or ``'parent_of'``
+        (optional)
+    :type type: string
+    :param comment: a comment about the relationship (optional)
+    :type comment: string
+
+    :returns: the updated relationship
+    :rtype: dictionary
+
+    '''
     model = context['model']
-    user = context['user']
     schema = context.get('schema') or ckan.logic.schema.default_update_relationship_schema()
 
-    id = data_dict['subject']
-    id2 = data_dict['object']
-    rel = data_dict['type']
+    id, id2, rel = _get_or_bust(data_dict, ['subject', 'object', 'type'])
 
     pkg1 = model.Package.get(id)
     pkg2 = model.Package.get(id2)
@@ -286,13 +369,12 @@ def package_relationship_update(context, data_dict):
     if not pkg2:
         return NotFound('Object package %r was not found.' % id2)
 
-    data, errors = validate(data_dict, schema, context)
-
+    data, errors = _validate(data_dict, schema, context)
     if errors:
         model.Session.rollback()
-        raise ValidationError(errors, error_summary(errors))
+        raise ValidationError(errors)
 
-    check_access('package_relationship_update', context, data_dict)
+    _check_access('package_relationship_update', context, data_dict)
 
     existing_rels = pkg1.get_relationships_with(pkg2, rel)
     if not existing_rels:
@@ -303,10 +385,26 @@ def package_relationship_update(context, data_dict):
     return _update_package_relationship(entity, comment, context)
 
 def group_update(context, data_dict):
+    '''Update a group.
+
+    You must be authorized to edit the group.
+
+    Plugins may change the parameters of this function depending on the value
+    of the group's ``type`` attribute, see the ``IGroupForm`` plugin interface.
+
+    For further parameters see ``group_create()``.
+
+    :param id: the name or id of the group to update
+    :type id: string
+
+    :returns: the updated group
+    :rtype: dictionary
+
+    '''
     model = context['model']
     user = context['user']
     session = context['session']
-    id = data_dict['id']
+    id = _get_or_bust(data_dict, 'id')
     parent = context.get('parent', None)
 
     group = model.Group.get(id)
@@ -323,12 +421,25 @@ def group_update(context, data_dict):
     except AttributeError:
         schema = group_plugin.form_to_db_schema()
 
-    check_access('group_update', context, data_dict)
+    _check_access('group_update', context, data_dict)
+
+    if 'api_version' not in context:
+        # old plugins do not support passing the schema so we need
+        # to ensure they still work
+        try:
+            group_plugin.check_data_dict(data_dict, schema)
+        except TypeError:
+            group_plugin.check_data_dict(data_dict)
+
+    data, errors = _validate(data_dict, schema, context)
+    log.debug('group_update validate_errs=%r user=%s group=%s data_dict=%r',
+              errors, context.get('user'),
+              context.get('group').name if context.get('group') else '',
+              data_dict)
 
-    data, errors = validate(data_dict, schema, context)
     if errors:
         session.rollback()
-        raise ValidationError(errors, error_summary(errors))
+        raise ValidationError(errors)
 
     rev = model.repo.new_revision()
     rev.author = user
@@ -347,10 +458,15 @@ def group_update(context, data_dict):
             current = session.query(model.Member).\
                filter(model.Member.table_id == group.id).\
                filter(model.Member.table_name == "group").all()
+            if current:
+                log.debug('Parents of group %s deleted: %r', group.name,
+                          [membership.group.name for membership in current])
             for c in current:
                 session.delete(c)
             member = model.Member(group=parent_group, table_id=group.id, table_name='group')
             session.add(member)
+            log.debug('Group %s is made child of group %s',
+                      group.name, parent_group.name)
 
 
     for item in plugins.PluginImplementations(plugins.IGroupController):
@@ -385,7 +501,7 @@ def group_update(context, data_dict):
             'defer_commit':True,
             'session': session
         }
-        get_action('activity_create')(activity_create_context, activity_dict,
+        _get_action('activity_create')(activity_create_context, activity_dict,
                 ignore_auth=True)
         # TODO: Also create an activity detail recording what exactly changed
         # in the group.
@@ -396,25 +512,37 @@ def group_update(context, data_dict):
     return model_dictize.group_dictize(group, context)
 
 def user_update(context, data_dict):
-    '''Updates the user\'s details'''
+    '''Update a user account.
 
+    Normal users can only update their own user accounts. Sysadmins can update
+    any user account.
+
+    For further parameters see ``user_create()``.
+
+    :param id: the name or id of the user to update
+    :type id: string
+
+    :returns: the updated user account
+    :rtype: dictionary
+
+    '''
     model = context['model']
     user = context['user']
     session = context['session']
     schema = context.get('schema') or ckan.logic.schema.default_update_user_schema()
-    id = data_dict['id']
+    id = _get_or_bust(data_dict, 'id')
 
     user_obj = model.User.get(id)
     context['user_obj'] = user_obj
     if user_obj is None:
         raise NotFound('User was not found.')
 
-    check_access('user_update', context, data_dict)
+    _check_access('user_update', context, data_dict)
 
-    data, errors = validate(data_dict, schema, context)
+    data, errors = _validate(data_dict, schema, context)
     if errors:
         session.rollback()
-        raise ValidationError(errors, error_summary(errors))
+        raise ValidationError(errors)
 
     user = model_save.user_dict_save(data, context)
 
@@ -429,7 +557,7 @@ def user_update(context, data_dict):
         'defer_commit':True,
         'session': session
     }
-    get_action('activity_create')(activity_create_context, activity_dict, ignore_auth=True)
+    _get_action('activity_create')(activity_create_context, activity_dict, ignore_auth=True)
     # TODO: Also create an activity detail recording what exactly changed in
     # the user.
 
@@ -438,6 +566,31 @@ def user_update(context, data_dict):
     return model_dictize.user_dictize(user, context)
 
 def task_status_update(context, data_dict):
+    '''Update a task status.
+
+    :param id: the id of the task status to update
+    :type id: string
+    :param entity_id:
+    :type entity_id: string
+    :param entity_type:
+    :type entity_type: string
+    :param task_type:
+    :type task_type: string
+    :param key:
+    :type key: string
+    :param value: (optional)
+    :type value:
+    :param state: (optional)
+    :type state:
+    :param last_updated: (optional)
+    :type last_updated:
+    :param error: (optional)
+    :type error:
+
+    :returns: the updated task status
+    :rtype: dictionary
+
+    '''
     model = context['model']
     session = model.meta.create_local_session()
     context['session'] = session
@@ -453,13 +606,12 @@ def task_status_update(context, data_dict):
         if task_status is None:
             raise NotFound(_('TaskStatus was not found.'))
 
-    check_access('task_status_update', context, data_dict)
-
-    data, errors = validate(data_dict, schema, context)
+    _check_access('task_status_update', context, data_dict)
 
+    data, errors = _validate(data_dict, schema, context)
     if errors:
         session.rollback()
-        raise ValidationError(errors, error_summary(errors))
+        raise ValidationError(errors)
 
     task_status = model_save.task_status_dict_save(data, context)
 
@@ -468,12 +620,22 @@ def task_status_update(context, data_dict):
     return model_dictize.task_status_dictize(task_status, context)
 
 def task_status_update_many(context, data_dict):
+    '''Update many task statuses at once.
+
+    :param data: the task_status dictionaries to update, for the format of task
+        status dictionaries see ``task_status_update()``
+    :type data: list of dictionaries
+
+    :returns: the updated task statuses
+    :rtype: list of dictionaries
+
+    '''
     results = []
     model = context['model']
     deferred = context.get('defer_commit')
     context['defer_commit'] = True
     for data in data_dict['data']:
-        results.append(get_action('task_status_update')(context, data))
+        results.append(_get_action('task_status_update')(context, data))
     if not deferred:
         context.pop('defer_commit')
     if not context.get('defer_commit'):
@@ -481,16 +643,32 @@ def task_status_update_many(context, data_dict):
     return {'results': results}
 
 def term_translation_update(context, data_dict):
+    '''Create or update a term translation.
+
+    You must be a sysadmin to create or update term translations.
+
+    :param term: the term to be translated, in the original language, e.g.
+        ``'romantic novel'``
+    :type term: string
+    :param term_translation: the translation of the term, e.g.
+        ``'Liebesroman'``
+    :type term_translation: string
+    :param lang_code: the language code of the translation, e.g. ``'de'``
+    :type lang_code: string
+
+    :returns: the newly created or updated term translation
+    :rtype: dictionary
+
+    '''
     model = context['model']
 
-    check_access('term_translation_update', context, data_dict)
+    _check_access('term_translation_update', context, data_dict)
 
     schema = {'term': [validators.not_empty, unicode],
               'term_translation': [validators.not_empty, unicode],
               'lang_code': [validators.not_empty, unicode]}
 
-    data, errors = validate(data_dict, schema, context)
-
+    data, errors = _validate(data_dict, schema, context)
     if errors:
         model.Session.rollback()
         raise ValidationError(errors)
@@ -515,6 +693,18 @@ def term_translation_update(context, data_dict):
     return data
 
 def term_translation_update_many(context, data_dict):
+    '''Create or update many term translations at once.
+
+    :param data: the term translation dictionaries to create or update,
+        for the format of term translation dictionaries see
+        ``term_translation_update()``
+    :type data: list of dictionaries
+
+    :returns: a dictionary with key ``'success'`` whose value is a string
+        stating how many term translations were updated
+    :rtype: string
+
+    '''
     model = context['model']
 
 
@@ -557,9 +747,9 @@ def package_update_rest(context, data_dict):
     context["allow_partial_update"] = False
     dictized_package = model_save.package_api_to_dict(data_dict, context)
 
-    check_access('package_update_rest', context, dictized_package)
+    _check_access('package_update_rest', context, dictized_package)
 
-    dictized_after = get_action('package_update')(context, dictized_package)
+    dictized_after = _get_action('package_update')(context, dictized_package)
 
     pkg = context['package']
 
@@ -570,15 +760,15 @@ def package_update_rest(context, data_dict):
 def group_update_rest(context, data_dict):
 
     model = context['model']
-    id = data_dict["id"]
+    id = _get_or_bust(data_dict, "id")
     group = model.Group.get(id)
     context["group"] = group
     context["allow_partial_update"] = True
     dictized_group = model_save.group_api_to_dict(data_dict, context)
 
-    check_access('group_update_rest', context, dictized_group)
+    _check_access('group_update_rest', context, dictized_group)
 
-    dictized_after = get_action('group_update')(context, dictized_group)
+    dictized_after = _get_action('group_update')(context, dictized_group)
 
     group = context['group']
 
@@ -587,6 +777,19 @@ def group_update_rest(context, data_dict):
     return group_dict
 
 def vocabulary_update(context, data_dict):
+    '''Update a tag vocabulary.
+
+    You must be a sysadmin to update vocabularies.
+
+    For further parameters see ``vocabulary_create()``.
+
+    :param id: the id of the vocabulary to update
+    :type id: string
+
+    :returns: the updated vocabulary
+    :rtype: dictionary
+
+    '''
     model = context['model']
 
     vocab_id = data_dict.get('id')
@@ -602,11 +805,10 @@ def vocabulary_update(context, data_dict):
         if data_dict['name'] == vocab.name:
             del data_dict['name']
 
-    check_access('vocabulary_update', context, data_dict)
+    _check_access('vocabulary_update', context, data_dict)
 
     schema = context.get('schema') or ckan.logic.schema.default_update_vocabulary_schema()
-    data, errors = validate(data_dict, schema, context)
-
+    data, errors = _validate(data_dict, schema, context)
     if errors:
         model.Session.rollback()
         raise ValidationError(errors)
@@ -631,22 +833,42 @@ def package_relationship_update_rest(context, data_dict):
     # these values.
     data_dict = logic.action.rename_keys(data_dict, key_map, destructive=True)
 
-    relationship_dict = get_action('package_relationship_update')(context, data_dict)
+    relationship_dict = _get_action('package_relationship_update')(context, data_dict)
 
     return relationship_dict
 
 def user_role_update(context, data_dict):
-    '''
-    For a named user (or authz group), set his/her authz roles on a domain_object.
+    '''Update a user or authorization group's roles for a domain object.
+
+    Either the ``user`` or the ``authorization_group`` parameter must be given.
+
+    You must be authorized to update the domain object.
+
+    To delete all of a user or authorization group's roles for domain object,
+    pass an empty list ``[]`` to the ``roles`` parameter.
+
+    :param user: the name or id of the user
+    :type user: string
+    :param authorization_group: the name or id of the authorization group
+    :type authorization_group: string
+    :param domain_object: the name or id of the domain object (e.g. a package,
+        group or authorization group)
+    :type domain_object: string
+    :param roles: the new roles, e.g. ``['editor']``
+    :type roles: list of strings
+
+    :returns: the updated roles of all users and authorization_groups for the
+        domain object
+    :rtype: dictionary
+
     '''
     model = context['model']
-    user = context['user'] # the current user, who is making the authz change
 
     new_user_ref = data_dict.get('user') # the user who is being given the new role
     new_authgroup_ref = data_dict.get('authorization_group') # the authgroup who is being given the new role
     if bool(new_user_ref) == bool(new_authgroup_ref):
         raise logic.ParameterError('You must provide either "user" or "authorization_group" parameter.')
-    domain_object_ref = data_dict['domain_object']
+    domain_object_ref = _get_or_bust(data_dict, 'domain_object')
     if not isinstance(data_dict['roles'], (list, tuple)):
         raise logic.ParameterError('Parameter "%s" must be of type: "%s"' % ('role', 'list'))
     desired_roles = set(data_dict['roles'])
@@ -669,11 +891,11 @@ def user_role_update(context, data_dict):
     domain_object = logic.action.get_domain_object(model, domain_object_ref)
     data_dict['id'] = domain_object.id
     if isinstance(domain_object, model.Package):
-        check_access('package_edit_permissions', context, data_dict)
+        _check_access('package_edit_permissions', context, data_dict)
     elif isinstance(domain_object, model.Group):
-        check_access('group_edit_permissions', context, data_dict)
+        _check_access('group_edit_permissions', context, data_dict)
     elif isinstance(domain_object, model.AuthorizationGroup):
-        check_access('authorization_group_edit_permissions', context, data_dict)
+        _check_access('authorization_group_edit_permissions', context, data_dict)
     # Todo: 'system' object
     else:
         raise logic.ParameterError('Not possible to update roles for domain object type %s' % type(domain_object))
@@ -681,7 +903,7 @@ def user_role_update(context, data_dict):
     # current_uors: in order to avoid either creating a role twice or
     # deleting one which is non-existent, we need to get the users\'
     # current roles (if any)
-    current_role_dicts = get_action('roles_show')(context, data_dict)['roles']
+    current_role_dicts = _get_action('roles_show')(context, data_dict)['roles']
     current_roles = set([role_dict['role'] for role_dict in current_role_dicts])
 
     # Whenever our desired state is different from our current state,
@@ -695,12 +917,21 @@ def user_role_update(context, data_dict):
     if not (current_roles == desired_roles):
         model.repo.commit_and_remove()
 
-    return get_action('roles_show')(context, data_dict)
+    return _get_action('roles_show')(context, data_dict)
 
 def user_role_bulk_update(context, data_dict):
-    '''
-    For a given domain_object, update authz roles that several users have on it.
-    To delete all roles for a user on a domain object, set {roles: []}.
+    '''Update the roles of many users or authorization groups for an object.
+
+    You must be authorized to update the domain object.
+
+    :param user_roles: the updated user roles, for the format of user role
+        dictionaries see ``user_role_update()``
+    :type user_roles: list of dictionaries
+
+    :returns: the updated roles of all users and authorization groups for the
+        domain object
+    :rtype: dictionary
+
     '''
     for user_or_authgroup in ('user', 'authorization_group'):
         # Collate all the roles for each user
@@ -718,4 +949,4 @@ def user_role_bulk_update(context, data_dict):
                              'roles': roles_by_user[user],
                              'domain_object': data_dict['domain_object']}
             user_role_update(context, uro_data_dict)
-    return get_action('roles_show')(context, data_dict)
+    return _get_action('roles_show')(context, data_dict)
