@@ -1,7 +1,8 @@
-from ckan.logic import check_access_old, NotFound
+import ckan.logic as logic
 from ckan.authz import Authorizer
 from ckan.lib.base import _
-from ckan.logic.auth import get_package_object, get_group_object, get_resource_object
+from ckan.logic.auth import (get_package_object, get_group_object,
+                            get_resource_object, get_related_object)
 
 
 def site_read(context, data_dict):
@@ -84,12 +85,16 @@ def package_show(context, data_dict):
     user = context.get('user')
     package = get_package_object(context, data_dict)
 
-    authorized = check_access_old(package, model.Action.READ, context)
+    authorized = logic.check_access_old(package, model.Action.READ, context)
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to read package %s') % (str(user),package.id)}
     else:
         return {'success': True}
 
+def related_show(context, data_dict=None):
+    return {'success': True}
+
+
 def resource_show(context, data_dict):
     model = context['model']
     user = context.get('user')
@@ -102,11 +107,11 @@ def resource_show(context, data_dict):
         .filter(model.ResourceGroup.id == resource.resource_group_id)
     pkg = query.first()
     if not pkg:
-        raise NotFound(_('No package found for this resource, cannot check auth.'))
-    
+        raise logic.NotFound(_('No package found for this resource, cannot check auth.'))
+
     pkg_dict = {'id': pkg.id}
     authorized = package_show(context, pkg_dict).get('success')
-    
+
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to read resource %s') % (str(user), resource.id)}
     else:
@@ -121,7 +126,7 @@ def group_show(context, data_dict):
     user = context.get('user')
     group = get_group_object(context, data_dict)
 
-    authorized =  check_access_old(group, model.Action.READ, context)
+    authorized =  logic.check_access_old(group, model.Action.READ, context)
     if not authorized:
         return {'success': False, 'msg': _('User %s not authorized to read group %s') % (str(user),group.id)}
     else:
@@ -154,6 +159,9 @@ def format_autocomplete(context, data_dict):
 def task_status_show(context, data_dict):
     return {'success': True}
 
+def resource_status_show(context, data_dict):
+    return {'success': True}
+
 ## Modifications for rest api
 
 def package_show_rest(context, data_dict):
