@@ -93,6 +93,15 @@ def related_update(context, data_dict):
     if not userobj or userobj.id != related.owner_id:
         return {'success': False, 'msg': _('Only the owner can update a related item')}
 
+    # Only sysadmins can change the featured field.
+    if ('featured' in data_dict and
+        data_dict['featured'] != related.featured and
+        not Authorizer().is_sysadmin(unicode(user))):
+
+        return {'success': False,
+                'msg': _('You must be a sysadmin to change a related item\'s '
+                         'featured field.')}
+
     return {'success': True}
 
 
@@ -141,7 +150,6 @@ def authorization_group_edit_permissions(context, data_dict):
         return {'success': True}
 
 def user_update(context, data_dict):
-    model = context['model']
     user = context['user']
     user_obj = get_user_object(context, data_dict)
 
@@ -180,7 +188,6 @@ def vocabulary_update(context, data_dict):
 
 def term_translation_update(context, data_dict):
 
-    model = context['model']
     user = context['user']
 
     if 'ignore_auth' in context and context['ignore_auth']:
