@@ -101,6 +101,17 @@ def user_id_exists(user_id, context):
         raise Invalid('%s: %s' % (_('Not found'), _('User')))
     return user_id
 
+def user_id_or_name_exists(user_id_or_name, context):
+    model = context['model']
+    session = context['session']
+    result = session.query(model.User).get(user_id_or_name)
+    if result:
+        return user_id_or_name
+    result = session.query(model.User).filter_by(name=user_id_or_name).first()
+    if not result:
+        raise Invalid('%s: %s' % (_('Not found'), _('User')))
+    return result.id
+
 def group_id_exists(group_id, context):
     """Raises Invalid if the given group_id does not exist in the model given
     in the context, otherwise returns the given group_id.
@@ -114,6 +125,20 @@ def group_id_exists(group_id, context):
         raise Invalid('%s: %s' % (_('Not found'), _('Group')))
     return group_id
 
+
+def related_id_exists(related_id, context):
+    """Raises Invalid if the given related_id does not exist in the model
+    given in the context, otherwise returns the given related_id.
+
+    """
+    model = context['model']
+    session = context['session']
+
+    result = session.query(model.Related).get(related_id)
+    if not result:
+        raise Invalid('%s: %s' % (_('Not found'), _('Related')))
+    return related_id
+
 def group_id_or_name_exists(reference, context):
     """
     Raises Invalid if a group identified by the name or id cannot be found.
@@ -141,11 +166,15 @@ object_id_validators = {
     'new package' : package_id_exists,
     'changed package' : package_id_exists,
     'deleted package' : package_id_exists,
+    'follow dataset' : package_id_exists,
     'new user' : user_id_exists,
     'changed user' : user_id_exists,
+    'follow user' : user_id_exists,
     'new group' : group_id_exists,
     'changed group' : group_id_exists,
     'deleted group' : group_id_exists,
+    'new related item': related_id_exists,
+    'deleted related item': related_id_exists
     }
 
 def object_id_validator(key, activity_dict, errors, context):
@@ -476,3 +505,23 @@ def tag_not_in_vocabulary(key, tag_dict, errors, context):
                 (tag_name, vocabulary_id))
     else:
         return
+
+def url_validator(key, data, errors, context):
+    """ Checks that the provided value (if it is present) is a valid URL """
+    import urlparse
+    import string
+
+    model = context['model']
+    session = context['session']
+
+    url = data.get(key, None)
+    if not url:
+        return
+
+    pieces = urlparse.urlparse(url)
+    if all([pieces.scheme, pieces.netloc]) and \
+       set(pieces.netloc) <= set(string.letters + string.digits + '-.') and \
+       pieces.scheme in ['http', 'https']:
+       return
+
+    errors[key].append(_('Please provide a valid URL'))
