@@ -1,44 +1,165 @@
-from __future__ import with_statement # necessary for python 2.5 support
+from __future__ import with_statement   # necessary for python 2.5 support
 import warnings
 import logging
+import re
+from datetime import datetime
 
-with warnings.catch_warnings():
-    warnings.filterwarnings('ignore', '.*Unbuilt egg.*')
-    from pylons import config
-from sqlalchemy import MetaData, __version__ as sqav
-from sqlalchemy.schema import Index
-from paste.deploy.converters import asbool
+import vdm.sqlalchemy
+from vdm.sqlalchemy.base import SQLAlchemySession
+from sqlalchemy import MetaData, __version__ as sqav, Table
+from sqlalchemy.util import OrderedDict
 
 import meta
-from domain_object import DomainObjectOperation
-from core import *
-from package import *
-from tag import *
-from package_mapping import *
-from user import user_table, User
-from authorization_group import *
-from group import *
-from group_extra import *
-from authz import *
-from package_extra import *
-from resource import *
-from tracking import *
-from rating import *
-from package_relationship import *
-from task_status import *
-from vocabulary import *
-from activity import *
-from related import *
-from term_translation import *
+from meta import (
+    Session,
+    engine_is_sqlite,
+)
+from core import (
+    System,
+    Revision,
+    State,
+    revision_table,
+)
+from package import (
+    Package,
+    PACKAGE_NAME_MIN_LENGTH,
+    PACKAGE_NAME_MAX_LENGTH,
+    PACKAGE_VERSION_MAX_LENGTH,
+    package_table,
+    package_revision_table,
+    PackageTagRevision,
+    PackageRevision,
+)
+from tag import (
+    Tag,
+    PackageTag,
+    MAX_TAG_LENGTH,
+    MIN_TAG_LENGTH,
+    tag_table,
+    package_tag_table,
+    package_tag_revision_table,
+)
+from user import (
+    User,
+    user_table,
+)
+from authz import (
+    NotRealUserException,
+    Enum,
+    Action,
+    Role,
+    RoleAction,
+    UserObjectRole,
+    PackageRole,
+    GroupRole,
+    AuthorizationGroupRole,
+    SystemRole,
+    PSEUDO_USER__VISITOR,
+    PSEUDO_USER__LOGGED_IN,
+    init_authz_const_data,
+    init_authz_configuration_data,
+    add_user_to_role,
+    add_authorization_group_to_role,
+    setup_user_roles,
+    setup_default_user_roles,
+    give_all_packages_default_user_roles,
+    user_has_role,
+    remove_user_from_role,
+    remove_authorization_group_from_role,
+    clear_user_roles,
+)
+from authorization_group import (
+    AuthorizationGroup,
+    AuthorizationGroupUser,
+    user_in_authorization_group,
+    add_user_to_authorization_group,
+    remove_user_from_authorization_group,
+)
+from group import (
+    Member,
+    Group,
+    member_revision_table,
+    group_table,
+    GroupRevision,
+    MemberRevision,
+    member_table,
+)
+from group_extra import (
+    GroupExtra,
+    group_extra_table,
+    GroupExtraRevision,
+)
+from package_extra import (
+    PackageExtra,
+    PackageExtraRevision,
+    package_extra_table,
+    extra_revision_table,
+)
+from resource import (
+    Resource,
+    ResourceGroup,
+    ResourceRevision,
+    DictProxy,
+    resource_group_table,
+    resource_table,
+    resource_revision_table,
+    ResourceGroupRevision,
+    resource_group_revision_table,
+)
+from tracking import (
+    tracking_summary_table,
+    TrackingSummary,
+)
+from rating import (
+    Rating,
+)
+from related import (
+    Related,
+    RelatedDataset,
+    related_dataset_table,
+    related_table,
+)
+from package_relationship import (
+    PackageRelationship,
+    package_relationship_table,
+    package_relationship_revision_table,
+)
+from task_status import (
+    TaskStatus,
+    task_status_table,
+)
+from vocabulary import (
+    Vocabulary,
+    VOCABULARY_NAME_MAX_LENGTH,
+    VOCABULARY_NAME_MIN_LENGTH,
+)
+from activity import (
+    Activity,
+    ActivityDetail,
+    activity_table,
+    activity_detail_table,
+)
+from term_translation import (
+    term_translation_table,
+)
+from follower import (
+    UserFollowingUser,
+    UserFollowingDataset,
+)
+from domain_object import (
+    DomainObjectOperation,
+    DomainObject,
+)
 import ckan.migration
-from ckan.lib.helpers import OrderedDict, datetime_to_date_str
-from vdm.sqlalchemy.base import SQLAlchemySession
 
 log = logging.getLogger(__name__)
 
+
+
 # set up in init_model after metadata is bound
 version_table = None
 
+
 def init_model(engine):
     '''Call me before using any of the tables or classes in the model'''
     meta.Session.remove()
@@ -49,17 +170,18 @@ def init_model(engine):
     # sqlalchemy migrate version table
     import sqlalchemy.exc
     try:
-        version_table = Table('migrate_version', metadata, autoload=True)
+        global version_table
+        version_table = Table('migrate_version', meta.metadata, autoload=True)
     except sqlalchemy.exc.NoSuchTableError:
         pass
 
 
-
 class Repository(vdm.sqlalchemy.Repository):
     migrate_repository = ckan.migration.__path__[0]
 
-    # note: tables_created value is not sustained between instantiations so
-    #       only useful for tests. The alternative is to use are_tables_created().
+    # note: tables_created value is not sustained between instantiations
+    #       so only useful for tests. The alternative is to use
+    #       are_tables_created().
     tables_created_and_initialised = False
 
     def init_db(self):
@@ -81,12 +203,13 @@ class Repository(vdm.sqlalchemy.Repository):
         else:
             if not self.tables_created_and_initialised:
                 self.upgrade_db()
-                ## make sure celery tables are made as celery only makes them after
-                ## adding a task
+                ## make sure celery tables are made as celery only makes
+                ## them after adding a task
                 try:
                     import ckan.lib.celery_app as celery_app
                     import celery.db.session as celery_session
-                    ##This creates the database tables it is a slight hack to celery.
+                    ## This creates the database tables it is a slight
+                    ## hack to celery.
                     backend = celery_app.celery.backend
                     celery_result_session = backend.ResultSession()
                     engine = celery_result_session.bind
@@ -99,12 +222,12 @@ class Repository(vdm.sqlalchemy.Repository):
         log.info('Database initialised')
 
     def clean_db(self):
-        metadata = MetaData(self.metadata.bind)
+        meta.metadata = MetaData(self.metadata.bind)
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', '.*(reflection|tsvector).*')
-            metadata.reflect()
+            meta.metadata.reflect()
 
-        metadata.drop_all()
+        meta.metadata.drop_all()
         self.tables_created_and_initialised = False
         log.info('Database tables dropped')
 
@@ -115,16 +238,16 @@ class Repository(vdm.sqlalchemy.Repository):
                          PSEUDO_USER__VISITOR):
             if not User.by_name(username):
                 user = User(name=username)
-                Session.add(user)
-        Session.flush() # so that these objects can be used
-                        # straight away
+                meta.Session.add(user)
+        meta.Session.flush()    # so that these objects can be used
+                                # straight away
         init_authz_const_data()
 
     def init_configuration_data(self):
         '''Default configuration, for when CKAN is first used out of the box.
         This state may be subsequently configured by the user.'''
         init_authz_configuration_data()
-        if Session.query(Revision).count() == 0:
+        if meta.Session.query(Revision).count() == 0:
             rev = Revision()
             rev.author = 'system'
             rev.message = u'Initialising the Repository'
@@ -193,8 +316,8 @@ class Repository(vdm.sqlalchemy.Repository):
         @param version: version to upgrade to (if None upgrade to latest)
         '''
         assert meta.engine.name in ('postgres', 'postgresql'), \
-            'Database migration - only Postgresql engine supported (not %s).' %\
-            meta.engine.name
+            'Database migration - only Postgresql engine supported (not %s).' \
+                % meta.engine.name
         import migrate.versioning.api as mig
         self.setup_migration_version_control()
         version_before = mig.db_version(self.metadata.bind, self.migrate_repository)
@@ -206,52 +329,53 @@ class Repository(vdm.sqlalchemy.Repository):
             log.info('CKAN database version remains as: %s', version_after)
 
         self.init_const_data()
-        
+
         ##this prints the diffs in a readable format
         ##import pprint
         ##from migrate.versioning.schemadiff import getDiffOfModelAgainstDatabase
         ##pprint.pprint(getDiffOfModelAgainstDatabase(self.metadata, self.metadata.bind).colDiffs)
 
     def are_tables_created(self):
-        metadata = MetaData(self.metadata.bind)
+        meta.metadata = MetaData(self.metadata.bind)
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', '.*(reflection|geometry).*')
-            metadata.reflect()
-        return bool(metadata.tables)
+            meta.metadata.reflect()
+        return bool(meta.metadata.tables)
 
     def purge_revision(self, revision, leave_record=False):
         '''Purge all changes associated with a revision.
 
-        @param leave_record: if True leave revision in existence but change message
-            to "PURGED: {date-time-of-purge}". If false delete revision object as
-            well.
+        @param leave_record: if True leave revision in existence but
+        change message to "PURGED: {date-time-of-purge}". If false
+        delete revision object as well.
 
         Summary of the Algorithm
         ------------------------
 
         1. list all RevisionObjects affected by this revision
         2. check continuity objects and cascade on everything else ?
-            1. crudely get all object revisions associated with this
-            2. then check whether this is the only revision and delete the
-            continuity object
+        3. crudely get all object revisions associated with this
+        4. then check whether this is the only revision and delete
+           the continuity object
+
+        5. ALTERNATIVELY delete all associated object revisions then
+           do a select on continutity to check which have zero
+           associated revisions (should only be these ...) '''
 
-            3. ALTERNATIVELY delete all associated object revisions then do a
-            select on continutity to check which have zero associated revisions
-            (should only be these ...)
-        '''
         to_purge = []
         SQLAlchemySession.setattr(self.session, 'revisioning_disabled', True)
         self.session.autoflush = False
         for o in self.versioned_objects:
             revobj = o.__revision_class__
-            items = self.session.query(revobj).filter_by(revision=revision).all()
+            items = self.session.query(revobj). \
+                    filter_by(revision=revision).all()
             for item in items:
                 continuity = item.continuity
 
-                if continuity.revision == revision: # need to change continuity
-                    trevobjs = self.session.query(revobj).join('revision').  filter(
-                            revobj.continuity==continuity
-                            ).order_by(Revision.timestamp.desc()).all()
+                if continuity.revision == revision:  # must change continuity
+                    trevobjs = self.session.query(revobj).join('revision'). \
+                            filter(revobj.continuity == continuity). \
+                            order_by(Revision.timestamp.desc()).all()
                     if len(trevobjs) == 0:
                         raise Exception('Should have at least one revision.')
                     if len(trevobjs) == 1:
@@ -263,8 +387,7 @@ class Repository(vdm.sqlalchemy.Repository):
                                 continue
                             if 'pending' not in obj.state:
                                 obj.current = True
-                                import datetime
-                                obj.expired_timestamp = datetime.datetime(9999, 12, 31)
+                                obj.expired_timestamp = datetime(9999, 12, 31)
                                 self.session.add(obj)
                                 break
                 # now delete revision object
@@ -272,15 +395,16 @@ class Repository(vdm.sqlalchemy.Repository):
             for cont in to_purge:
                 self.session.delete(cont)
         if leave_record:
-            import datetime
-            revision.message = u'PURGED: %s' % datetime.datetime.now()
+            revision.message = u'PURGED: %s' % datetime.now()
         else:
             self.session.delete(revision)
         self.commit_and_remove()
 
 
-repo = Repository(metadata, Session,
-        versioned_objects=[Package, PackageTag, Resource, ResourceGroup, PackageExtra, Member, Group]
+repo = Repository(meta.metadata, meta.Session,
+                  versioned_objects=[Package, PackageTag, Resource,
+                                     ResourceGroup, PackageExtra, Member,
+                                     Group]
         )
 
 
@@ -296,6 +420,7 @@ def _get_packages(self):
 
     return list(pkgs)
 
+
 def _get_groups(self):
     changes = repo.list_changes(self)
     groups = set()
@@ -307,24 +432,27 @@ def _get_groups(self):
                 groups.add(non_group_rev.continuity.group)
     return list(groups)
 
+
 # could set this up directly on the mapper?
 def _get_revision_user(self):
     username = unicode(self.author)
-    user = Session.query(User).filter_by(name=username).first()
+    user = meta.Session.query(User).filter_by(name=username).first()
     return user
 
 Revision.packages = property(_get_packages)
 Revision.groups = property(_get_groups)
 Revision.user = property(_get_revision_user)
 
-def revision_as_dict(revision, include_packages=True, include_groups=True,ref_package_by='name'):
+
+def revision_as_dict(revision, include_packages=True, include_groups=True,
+                     ref_package_by='name'):
     revision_dict = OrderedDict((
         ('id', revision.id),
-        ('timestamp', datetime_to_date_str(revision.timestamp)),
+        ('timestamp', revision.timestamp.isoformat()),
         ('message', revision.message),
         ('author', revision.author),
         ('approved_timestamp',
-         datetime_to_date_str(revision.approved_timestamp) \
+         revision.approved_timestamp.isoformat() \
          if revision.approved_timestamp else None),
         ))
     if include_packages:
@@ -336,7 +464,8 @@ def revision_as_dict(revision, include_packages=True, include_groups=True,ref_pa
 
     return revision_dict
 
+
 def is_id(id_string):
     '''Tells the client if the string looks like a revision id or not'''
-    import re
-    return bool(re.match('^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$', id_string))
+    reg_ex = '^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$'
+    return bool(re.match(reg_ex, id_string))
