@@ -1,43 +1,41 @@
 import datetime
 
-from sqlalchemy import orm
+from sqlalchemy import orm, types, Column, Table, ForeignKey
 
-from meta import *
-from types import make_uuid
-from types import JsonDictType
-from core import *
-from package import *
+import meta
+import types as _types
+import domain_object
 
-__all__ = ['Activity', 'activity_table', 
+__all__ = ['Activity', 'activity_table',
            'ActivityDetail', 'activity_detail_table',
            ]
 
 activity_table = Table(
-    'activity', metadata,
-    Column('id', types.UnicodeText, primary_key=True, default=make_uuid),
+    'activity', meta.metadata,
+    Column('id', types.UnicodeText, primary_key=True, default=_types.make_uuid),
     Column('timestamp', types.DateTime),
     Column('user_id', types.UnicodeText),
     Column('object_id', types.UnicodeText),
     Column('revision_id', types.UnicodeText),
     Column('activity_type', types.UnicodeText),
-    Column('data', JsonDictType),
+    Column('data', _types.JsonDictType),
     )
 
 activity_detail_table = Table(
-    'activity_detail', metadata,
-    Column('id', types.UnicodeText, primary_key=True, default=make_uuid),
+    'activity_detail', meta.metadata,
+    Column('id', types.UnicodeText, primary_key=True, default=_types.make_uuid),
     Column('activity_id', types.UnicodeText, ForeignKey('activity.id')),
     Column('object_id', types.UnicodeText),
     Column('object_type', types.UnicodeText),
     Column('activity_type', types.UnicodeText),
-    Column('data', JsonDictType),
+    Column('data', _types.JsonDictType),
     )
 
-class Activity(DomainObject):
+class Activity(domain_object.DomainObject):
 
     def __init__(self, user_id, object_id, revision_id, activity_type,
             data=None):
-        self.id = make_uuid()
+        self.id = _types.make_uuid()
         self.timestamp = datetime.datetime.now()
         self.user_id = user_id
         self.object_id = object_id
@@ -48,9 +46,9 @@ class Activity(DomainObject):
         else:
             self.data = data
 
-mapper(Activity, activity_table)
+meta.mapper(Activity, activity_table)
 
-class ActivityDetail(DomainObject):
+class ActivityDetail(domain_object.DomainObject):
 
     def __init__(self, activity_id, object_id, object_type, activity_type,
             data=None):
@@ -63,6 +61,6 @@ class ActivityDetail(DomainObject):
         else:
             self.data = data
 
-mapper(ActivityDetail, activity_detail_table, properties = {
+meta.mapper(ActivityDetail, activity_detail_table, properties = {
     'activity':orm.relation ( Activity, backref=orm.backref('activity_detail'))
     })
