@@ -2,14 +2,30 @@
 doc/authorization.rst.
 
 '''
-from meta import *
-from core import *
-from package import *
-from group import Group
-from types import make_uuid
-from user import User
-from core import System
-from authorization_group import AuthorizationGroup, authorization_group_table
+import simplejson as json
+import weakref
+
+from sqlalchemy import orm, types, Column, Table, ForeignKey
+from pylons import config
+
+import meta
+import core
+import domain_object
+import package as _package
+import group
+import user as _user
+import authorization_group as auth_group
+import types as _types
+
+__all__ = ['NotRealUserException', 'Enum', 'Action', 'Role', 'RoleAction',
+           'UserObjectRole', 'PackageRole', 'GroupRole',
+           'AuthorizationGroupRole', 'SystemRole', 'PSEUDO_USER__VISITOR',
+           'PSEUDO_USER__LOGGED_IN', 'init_authz_const_data',
+           'init_authz_configuration_data', 'add_user_to_role',
+           'add_authorization_group_to_role', 'setup_user_roles',
+           'setup_default_user_roles', 'give_all_packages_default_user_roles',
+           'user_has_role', 'remove_user_from_role',
+           'remove_authorization_group_from_role', 'clear_user_roles']
 
 PSEUDO_USER__LOGGED_IN = u'logged_in'
 PSEUDO_USER__VISITOR = u'visitor'
@@ -86,42 +102,42 @@ default_role_actions = [
 ## ======================================
 ## Table Definitions
 
-role_action_table = Table('role_action', metadata,
-           Column('id', UnicodeText, primary_key=True, default=make_uuid),
-           Column('role', UnicodeText),
-           Column('context', UnicodeText, nullable=False),
-           Column('action', UnicodeText),
+role_action_table = Table('role_action', meta.metadata,
+           Column('id', types.UnicodeText, primary_key=True, default=_types.make_uuid),
+           Column('role', types.UnicodeText),
+           Column('context', types.UnicodeText, nullable=False),
+           Column('action', types.UnicodeText),
            )
 
-user_object_role_table = Table('user_object_role', metadata,
-           Column('id', UnicodeText, primary_key=True, default=make_uuid),
-           Column('user_id', UnicodeText, ForeignKey('user.id'), nullable=True),
-           Column('authorized_group_id', UnicodeText, ForeignKey('authorization_group.id'), nullable=True),
-           Column('context', UnicodeText, nullable=False), # stores subtype
-           Column('role', UnicodeText)
+user_object_role_table = Table('user_object_role', meta.metadata,
+           Column('id', types.UnicodeText, primary_key=True, default=_types.make_uuid),
+           Column('user_id', types.UnicodeText, ForeignKey('user.id'), nullable=True),
+           Column('authorized_group_id', types.UnicodeText, ForeignKey('authorization_group.id'), nullable=True),
+           Column('context', types.UnicodeText, nullable=False), # stores subtype
+           Column('role', types.UnicodeText)
            )
 
-package_role_table = Table('package_role', metadata,
-           Column('user_object_role_id', UnicodeText, ForeignKey('user_object_role.id'), primary_key=True),
-           Column('package_id', UnicodeText, ForeignKey('package.id')),
+package_role_table = Table('package_role', meta.metadata,
+           Column('user_object_role_id', types.UnicodeText, ForeignKey('user_object_role.id'), primary_key=True),
+           Column('package_id', types.UnicodeText, ForeignKey('package.id')),
            )
 
-group_role_table = Table('group_role', metadata,
-           Column('user_object_role_id', UnicodeText, ForeignKey('user_object_role.id'), primary_key=True),
-           Column('group_id', UnicodeText, ForeignKey('group.id')),
+group_role_table = Table('group_role', meta.metadata,
+           Column('user_object_role_id', types.UnicodeText, ForeignKey('user_object_role.id'), primary_key=True),
+           Column('group_id', types.UnicodeText, ForeignKey('group.id')),
            )
            
-authorization_group_role_table = Table('authorization_group_role', metadata,
-           Column('user_object_role_id', UnicodeText, ForeignKey('user_object_role.id'), primary_key=True),
-           Column('authorization_group_id', UnicodeText, ForeignKey('authorization_group.id')),
+authorization_group_role_table = Table('authorization_group_role', meta.metadata,
+           Column('user_object_role_id', types.UnicodeText, ForeignKey('user_object_role.id'), primary_key=True),
+           Column('authorization_group_id', types.UnicodeText, ForeignKey('authorization_group.id')),
            )
 
-system_role_table = Table('system_role', metadata,
-           Column('user_object_role_id', UnicodeText, ForeignKey('user_object_role.id'), primary_key=True),
+system_role_table = Table('system_role', meta.metadata,
+           Column('user_object_role_id', types.UnicodeText, ForeignKey('user_object_role.id'), primary_key=True),
            )
 
 
-class RoleAction(DomainObject):
+class RoleAction(domain_object.DomainObject):
     def __repr__(self):
         return '<%s role="%s" action="%s" context="%s">' % \
                (self.__class__.__name__, self.role, self.action, self.context)
@@ -130,7 +146,7 @@ class RoleAction(DomainObject):
 # dictionary mapping protected objects (e.g. Package) to related ObjectRole
 protected_objects = {}
 
-class UserObjectRole(DomainObject):
+class UserObjectRole(domain_object.DomainObject):
     name = None
     protected_object = None
 
@@ -156,19 +172,19 @@ class UserObjectRole(DomainObject):
 
     @classmethod
     def user_has_role(cls, user, role, domain_obj):
-        assert isinstance(user, User), user
+        assert isinstance(user, _user.User), user
         q = cls._user_query(user, role, domain_obj)
         return q.count() == 1
         
     @classmethod
     def authorization_group_has_role(cls, authorized_group, role, domain_obj):
-        assert isinstance(authorized_group, AuthorizationGroup), authorized_group
+        assert isinstance(authorized_group, auth_group.AuthorizationGroup), authorized_group
         q = cls._authorized_group_query(authorized_group, role, domain_obj)
         return q.count() == 1
         
     @classmethod
     def _user_query(cls, user, role, domain_obj):
-        q = Session.query(cls).filter_by(role=role)
+        q = meta.Session.query(cls).filter_by(role=role)
         # some protected objects are not "contextual"
         if cls.name is not None:
             # e.g. filter_by(package=domain_obj)
@@ -178,7 +194,7 @@ class UserObjectRole(DomainObject):
     
     @classmethod
     def _authorized_group_query(cls, authorized_group, role, domain_obj):
-        q = Session.query(cls).filter_by(role=role)
+        q = meta.Session.query(cls).filter_by(role=role)
         # some protected objects are not "contextual"
         if cls.name is not None:
             # e.g. filter_by(package=domain_obj)
@@ -201,7 +217,7 @@ class UserObjectRole(DomainObject):
         objectrole = cls(role=role, user=user)
         if cls.name is not None:
             setattr(objectrole, cls.name, domain_obj)
-        Session.add(objectrole)
+        meta.Session.add(objectrole)
          
     @classmethod
     def add_authorization_group_to_role(cls, authorization_group, role, domain_obj):
@@ -216,26 +232,26 @@ class UserObjectRole(DomainObject):
         objectrole = cls(role=role, authorized_group=authorization_group)
         if cls.name is not None:
             setattr(objectrole, cls.name, domain_obj)
-        Session.add(objectrole)
+        meta.Session.add(objectrole)
 
     @classmethod
     def remove_user_from_role(cls, user, role, domain_obj):
         q = cls._user_query(user, role, domain_obj)
         for uo_role in q.all():
-            Session.delete(uo_role)
-        Session.commit()
-        Session.remove()
+            meta.Session.delete(uo_role)
+        meta.Session.commit()
+        meta.Session.remove()
 
     @classmethod
     def remove_authorization_group_from_role(cls, authorization_group, role, domain_obj):
         q = cls._authorized_group_query(authorization_group, role, domain_obj)
         for ago_role in q.all():
-            Session.delete(ago_role)
-        Session.commit()
-        Session.remove()
+            meta.Session.delete(ago_role)
+        meta.Session.commit()
+        meta.Session.remove()
 
 class PackageRole(UserObjectRole):
-    protected_object = Package
+    protected_object = _package.Package
     name = 'package'
 
     def __repr__(self):
@@ -251,7 +267,7 @@ class PackageRole(UserObjectRole):
 protected_objects[PackageRole.protected_object] = PackageRole
 
 class GroupRole(UserObjectRole):
-    protected_object = Group
+    protected_object = group.Group
     name = 'group'
 
     def __repr__(self):
@@ -267,7 +283,7 @@ class GroupRole(UserObjectRole):
 protected_objects[GroupRole.protected_object] = GroupRole
 
 class AuthorizationGroupRole(UserObjectRole):
-    protected_object = AuthorizationGroup
+    protected_object = auth_group.AuthorizationGroup
     name = 'authorization_group'
 
     def __repr__(self):
@@ -283,7 +299,7 @@ class AuthorizationGroupRole(UserObjectRole):
 protected_objects[AuthorizationGroupRole.protected_object] = AuthorizationGroupRole
 
 class SystemRole(UserObjectRole):
-    protected_object = System
+    protected_object = core.System
     name = None
 protected_objects[SystemRole.protected_object] = SystemRole
 
@@ -319,9 +335,9 @@ def remove_authorization_group_from_role(authorization_group, role, domain_obj):
     objectrole.remove_authorization_group_from_role(authorization_group, role, domain_obj)
     
 def init_authz_configuration_data():
-    setup_default_user_roles(System())
-    Session.commit()
-    Session.remove()
+    setup_default_user_roles(core.System())
+    meta.Session.commit()
+    meta.Session.remove()
     
 def init_authz_const_data():
     '''Setup all default role-actions.
@@ -333,44 +349,44 @@ def init_authz_const_data():
 
     '''
     for role, action in default_role_actions:
-        ra = Session.query(RoleAction).filter_by(role=role, action=action).first()
+        ra = meta.Session.query(RoleAction).filter_by(role=role, action=action).first()
         if ra is not None: continue
         ra = RoleAction(role=role, context=u'', action=action)
-        Session.add(ra)
-    Session.commit()
-    Session.remove()
+        meta.Session.add(ra)
+    meta.Session.commit()
+    meta.Session.remove()
 
 ## TODO: this should be in ckan/authz.py
-def setup_user_roles(domain_object, visitor_roles, logged_in_roles, admins=[]):
+def setup_user_roles(_domain_object, visitor_roles, logged_in_roles, admins=[]):
     '''NB: leaves caller to commit change'''
     assert type(admins) == type([])
     admin_roles = [Role.ADMIN]
-    visitor = User.by_name(PSEUDO_USER__VISITOR)
+    visitor = _user.User.by_name(PSEUDO_USER__VISITOR)
     assert visitor
     for role in visitor_roles:
-        add_user_to_role(visitor, role, domain_object)
-    logged_in = User.by_name(PSEUDO_USER__LOGGED_IN)
+        add_user_to_role(visitor, role, _domain_object)
+    logged_in = _user.User.by_name(PSEUDO_USER__LOGGED_IN)
     assert logged_in
     for role in logged_in_roles:
-        add_user_to_role(logged_in, role, domain_object)
+        add_user_to_role(logged_in, role, _domain_object)
     for admin in admins:
         # not sure if admin would reasonably by None
         if admin is not None:
-            assert isinstance(admin, User), admin
+            assert isinstance(admin, _user.User), admin
             if admin.name in (PSEUDO_USER__LOGGED_IN, PSEUDO_USER__VISITOR):
                 raise NotRealUserException('Invalid user for domain object admin %r' % admin.name)
             for role in admin_roles:
-                add_user_to_role(admin, role, domain_object)
+                add_user_to_role(admin, role, _domain_object)
 
 def give_all_packages_default_user_roles():
     # if this command gives an exception, you probably
     # forgot to do 'paster db init'
-    pkgs = Session.query(Package).all()
+    pkgs = meta.Session.query(_package.Package).all()
 
     for pkg in pkgs:
         print pkg
         # weird - should already be in session but complains w/o this
-        Session.add(pkg)
+        meta.Session.add(pkg)
         if len(pkg.roles) > 0:
             print 'Skipping (already has roles): %s' % pkg.name
             continue
@@ -380,7 +396,7 @@ def give_all_packages_default_user_roles():
         for rev in revs:
             if rev.revision.author:
                 # rev author is not Unicode!!
-                user = User.by_name(unicode(rev.revision.author))
+                user = _user.User.by_name(unicode(rev.revision.author))
                 if user:
                     admins.append(user)
         # remove duplicates
@@ -391,20 +407,18 @@ def give_all_packages_default_user_roles():
 
 # default user roles - used when the config doesn\'t specify them
 default_default_user_roles = {
-    'Package': {"visitor": ["editor"], "logged_in": ["editor"]},
+    'Package': {"visitor": ["reader"], "logged_in": ["reader"]},
     'Group': {"visitor": ["reader"], "logged_in": ["reader"]},
-    'System': {"visitor": ["anon_editor"], "logged_in": ["editor"]},
+    'System': {"visitor": ["reader"], "logged_in": ["editor"]},
     'AuthorizationGroup': {"visitor": ["reader"], "logged_in": ["reader"]},
     }
 
 global _default_user_roles_cache
-_default_user_roles_cache = {}
+_default_user_roles_cache = weakref.WeakKeyDictionary()
 
-def get_default_user_roles(domain_object):
+def get_default_user_roles(_domain_object):
     # TODO: Should this func go in lib rather than model now?
-    from ckan.lib.helpers import json
-    from pylons import config
-    def _get_default_user_roles(domain_object):
+    def _get_default_user_roles(_domain_object):
         config_key = 'ckan.default_roles.%s' % obj_type
         user_roles_json = config.get(config_key)
         if user_roles_json is None:
@@ -413,59 +427,59 @@ def get_default_user_roles(domain_object):
             user_roles_str = json.loads(user_roles_json) if user_roles_json else {}
         unknown_keys = set(user_roles_str.keys()) - set(('visitor', 'logged_in'))
         assert not unknown_keys, 'Auth config for %r has unknown key %r' % \
-               (domain_object, unknown_keys)
+               (_domain_object, unknown_keys)
         user_roles_ = {}
         for user in ('visitor', 'logged_in'):
             roles_str = user_roles_str.get(user, [])
             user_roles_[user] = [getattr(Role, role_str.upper()) for role_str in roles_str]
         return user_roles_
-    obj_type = domain_object.__class__.__name__
+    obj_type = _domain_object.__class__.__name__
     global _default_user_roles_cache
-    if not _default_user_roles_cache.has_key(domain_object):
-        _default_user_roles_cache[domain_object] = _get_default_user_roles(domain_object)
-    return _default_user_roles_cache[domain_object]
+    if not _default_user_roles_cache.has_key(_domain_object):
+        _default_user_roles_cache[_domain_object] = _get_default_user_roles(_domain_object)
+    return _default_user_roles_cache[_domain_object]
         
-def setup_default_user_roles(domain_object, admins=[]):
+def setup_default_user_roles(_domain_object, admins=[]):
     ''' sets up roles for visitor, logged-in user and any admins provided
     @param admins - a list of User objects
     NB: leaves caller to commit change.
     '''
-    assert isinstance(domain_object, (Package, Group, System, AuthorizationGroup)), domain_object
+    assert isinstance(_domain_object, (_package.Package, group.Group, core.System, auth_group.AuthorizationGroup)), _domain_object
     assert isinstance(admins, list)
-    user_roles_ = get_default_user_roles(domain_object)
-    setup_user_roles(domain_object,
+    user_roles_ = get_default_user_roles(_domain_object)
+    setup_user_roles(_domain_object,
                      user_roles_['visitor'],
                      user_roles_['logged_in'],
                      admins)
 
-def clear_user_roles(domain_object):
-    assert isinstance(domain_object, DomainObject)
-    if isinstance(domain_object, Package):
-        q = Session.query(PackageRole).filter_by(package=domain_object)
-    elif isinstance(domain_object, Group):
-        q = Session.query(GroupRole).filter_by(group=domain_object)
+def clear_user_roles(_domain_object):
+    assert isinstance(_domain_object, domain_object.DomainObject)
+    if isinstance(_domain_object, _package.Package):
+        q = meta.Session.query(PackageRole).filter_by(package=_domain_object)
+    elif isinstance(_domain_object, group.Group):
+        q = meta.Session.query(GroupRole).filter_by(group=_domain_object)
     else:
         raise NotImplementedError()
     user_roles = q.all()
     for user_role in user_roles:
-        Session.delete(user_role)
+        meta.Session.delete(user_role)
 
 
 ## ======================================
 ## Mappers
 
-mapper(RoleAction, role_action_table)
+meta.mapper(RoleAction, role_action_table)
        
-mapper(UserObjectRole, user_object_role_table,
+meta.mapper(UserObjectRole, user_object_role_table,
     polymorphic_on=user_object_role_table.c.context,
     polymorphic_identity=u'user_object',
     properties={
-        'user': orm.relation(User,
+        'user': orm.relation(_user.User,
             backref=orm.backref('roles',
                 cascade='all, delete, delete-orphan'
             )
         ),
-        'authorized_group': orm.relation(AuthorizationGroup,
+        'authorized_group': orm.relation(auth_group.AuthorizationGroup,
             backref=orm.backref('authorized_roles',
                 cascade='all, delete, delete-orphan'
             )
@@ -474,10 +488,10 @@ mapper(UserObjectRole, user_object_role_table,
     order_by=[user_object_role_table.c.id],
 )
 
-mapper(PackageRole, package_role_table, inherits=UserObjectRole,
-    polymorphic_identity=unicode(Package.__name__),
+meta.mapper(PackageRole, package_role_table, inherits=UserObjectRole,
+    polymorphic_identity=unicode(_package.Package.__name__),
     properties={
-        'package': orm.relation(Package,
+        'package': orm.relation(_package.Package,
              backref=orm.backref('roles',
              cascade='all, delete, delete-orphan'
              )
@@ -486,10 +500,10 @@ mapper(PackageRole, package_role_table, inherits=UserObjectRole,
     order_by=[package_role_table.c.user_object_role_id],
 )
 
-mapper(GroupRole, group_role_table, inherits=UserObjectRole,
-       polymorphic_identity=unicode(Group.__name__),
+meta.mapper(GroupRole, group_role_table, inherits=UserObjectRole,
+       polymorphic_identity=unicode(group.Group.__name__),
        properties={
-            'group': orm.relation(Group,
+            'group': orm.relation(group.Group,
                  backref=orm.backref('roles',
                  cascade='all, delete, delete-orphan'
                  ),
@@ -498,12 +512,12 @@ mapper(GroupRole, group_role_table, inherits=UserObjectRole,
     order_by=[group_role_table.c.user_object_role_id],
 )
 
-mapper(AuthorizationGroupRole, authorization_group_role_table, inherits=UserObjectRole,
-       polymorphic_identity=unicode(AuthorizationGroup.__name__),
+meta.mapper(AuthorizationGroupRole, authorization_group_role_table, inherits=UserObjectRole,
+       polymorphic_identity=unicode(auth_group.AuthorizationGroup.__name__),
        properties={
-            'authorization_group': orm.relation(AuthorizationGroup,
+            'authorization_group': orm.relation(auth_group.AuthorizationGroup,
                  backref=orm.backref('roles',
-                    primaryjoin=authorization_group_table.c.id==authorization_group_role_table.c.authorization_group_id,
+                    primaryjoin=auth_group.authorization_group_table.c.id==authorization_group_role_table.c.authorization_group_id,
                     cascade='all, delete, delete-orphan'
                  ),
             )
@@ -511,7 +525,7 @@ mapper(AuthorizationGroupRole, authorization_group_role_table, inherits=UserObje
     order_by=[authorization_group_role_table.c.user_object_role_id],
 )
 
-mapper(SystemRole, system_role_table, inherits=UserObjectRole,
-       polymorphic_identity=unicode(System.__name__),
+meta.mapper(SystemRole, system_role_table, inherits=UserObjectRole,
+       polymorphic_identity=unicode(core.System.__name__),
        order_by=[system_role_table.c.user_object_role_id],
 )
