@@ -1,16 +1,22 @@
+import datetime
+
 from sqlalchemy.util import OrderedDict
 from sqlalchemy.ext.orderinglist import ordering_list
 from sqlalchemy import orm
 from pylons import config
 import vdm.sqlalchemy
-import datetime
+import vdm.sqlalchemy.stateful
+from sqlalchemy import types, Column, Table, ForeignKey, and_
 
-from meta import *
-from types import make_uuid, JsonDictType
-from core import *
-from package import *
-from ckan.model import extension
-from ckan.model.activity import ActivityDetail
+import meta
+import core
+import package as _package
+import types as _types
+import extension
+import activity
+import domain_object
+import tracking as _tracking
+import ckan.lib.dictization
 
 __all__ = ['Resource', 'resource_table',
            'ResourceGroup', 'resource_group_table',
@@ -27,8 +33,8 @@ CORE_RESOURCE_COLUMNS = ['url', 'format', 'description', 'hash', 'name',
 
 ##formally package_resource
 resource_table = Table(
-    'resource', metadata,
-    Column('id', types.UnicodeText, primary_key=True, default=make_uuid),
+    'resource', meta.metadata,
+    Column('id', types.UnicodeText, primary_key=True, default=_types.make_uuid),
     Column('resource_group_id', types.UnicodeText, ForeignKey('resource_group.id')),
     Column('url', types.UnicodeText, nullable=False),
     Column('format', types.UnicodeText),
@@ -48,33 +54,33 @@ resource_table = Table(
     Column('webstore_url', types.UnicodeText),
     Column('webstore_last_updated', types.DateTime),
     
-    Column('extras', JsonDictType),
+    Column('extras', _types.JsonDictType),
     )
 
 resource_group_table = Table(
-    'resource_group', metadata,
-    Column('id', types.UnicodeText, primary_key=True, default=make_uuid),
+    'resource_group', meta.metadata,
+    Column('id', types.UnicodeText, primary_key=True, default=_types.make_uuid),
     Column('package_id', types.UnicodeText, ForeignKey('package.id')),
     Column('label', types.UnicodeText),
     Column('sort_order', types.UnicodeText),
-    Column('extras', JsonDictType),
+    Column('extras', _types.JsonDictType),
     )
 
 vdm.sqlalchemy.make_table_stateful(resource_table)
-resource_revision_table = make_revisioned_table(resource_table)
+resource_revision_table = core.make_revisioned_table(resource_table)
 
 vdm.sqlalchemy.make_table_stateful(resource_group_table)
-resource_group_revision_table = make_revisioned_table(resource_group_table)
+resource_group_revision_table = core.make_revisioned_table(resource_group_table)
 
 class Resource(vdm.sqlalchemy.RevisionedObjectMixin,
                vdm.sqlalchemy.StatefulObjectMixin,
-               DomainObject):
+               domain_object.DomainObject):
     extra_columns = None
     def __init__(self, resource_group_id=None, url=u'', 
                  format=u'', description=u'', hash=u'',
                  extras=None,
                  **kwargs):
-        self.id = make_uuid()
+        self.id = _types.make_uuid()
         if resource_group_id:
             self.resource_group_id = resource_group_id
         self.url = url
@@ -109,15 +115,14 @@ class Resource(vdm.sqlalchemy.RevisionedObjectMixin,
             _dict[k] = v
         if self.resource_group and not core_columns_only:
             _dict["package_id"] = self.resource_group.package_id
-        import ckan.model as model
-        tracking = model.TrackingSummary.get_for_resource(self.url)
+        tracking = _tracking.TrackingSummary.get_for_resource(self.url)
         _dict['tracking_summary'] = tracking
         return _dict
 
     @classmethod
     def get(cls, reference):
         '''Returns a resource object referenced by its name or id.'''
-        query = Session.query(ResourceRevision).filter(ResourceRevision.id==reference)
+        query = meta.Session.query(ResourceRevision).filter(ResourceRevision.id==reference)
         query = query.filter(and_(
             ResourceRevision.state == u'active', ResourceRevision.current == True
         ))
@@ -147,7 +152,6 @@ class Resource(vdm.sqlalchemy.RevisionedObjectMixin,
 
     def activity_stream_detail(self, activity_id, activity_type):
         import ckan.model as model
-        import ckan.lib.dictization
 
         # Handle 'deleted' resources.
         # When the user marks a resource as deleted this comes through here as
@@ -158,12 +162,12 @@ class Resource(vdm.sqlalchemy.RevisionedObjectMixin,
 
         res_dict = ckan.lib.dictization.table_dictize(self,
                 context={'model':model})
-        return ActivityDetail(activity_id, self.id, u"Resource", activity_type,
+        return activity.ActivityDetail(activity_id, self.id, u"Resource", activity_type,
                 {'resource': res_dict})
 
 class ResourceGroup(vdm.sqlalchemy.RevisionedObjectMixin,
                vdm.sqlalchemy.StatefulObjectMixin,
-               DomainObject):
+               domain_object.DomainObject):
     extra_columns = None
     def __init__(self, package_id=None, sort_order=u'', label=u'',
                  extras=None, **kwargs):
@@ -172,6 +176,7 @@ class ResourceGroup(vdm.sqlalchemy.RevisionedObjectMixin,
         self.sort_order = sort_order
         self.label = label
         self.extras = extras or {}
+        self.state = 'active'
 
         extra_columns = self.get_extra_columns()
         for field in extra_columns:
@@ -210,7 +215,7 @@ class ResourceGroup(vdm.sqlalchemy.RevisionedObjectMixin,
 
 ## Mappers
 
-mapper(Resource, resource_table, properties={
+meta.mapper(Resource, resource_table, properties={
     'resource_group':orm.relation(ResourceGroup,
         # all resources including deleted
         # formally package_resources_all
@@ -228,8 +233,8 @@ mapper(Resource, resource_table, properties={
 )
 
 
-mapper(ResourceGroup, resource_group_table, properties={
-    'package':orm.relation(Package,
+meta.mapper(ResourceGroup, resource_group_table, properties={
+    'package':orm.relation(_package.Package,
         # all resources including deleted
         backref=orm.backref('resource_groups_all',
                             cascade='all, delete, delete-orphan',
@@ -245,49 +250,20 @@ mapper(ResourceGroup, resource_group_table, properties={
 
 ## VDM
 
-vdm.sqlalchemy.modify_base_object_mapper(Resource, Revision, State)
+vdm.sqlalchemy.modify_base_object_mapper(Resource, core.Revision, core.State)
 ResourceRevision = vdm.sqlalchemy.create_object_version(
-    mapper, Resource, resource_revision_table)
+    meta.mapper, Resource, resource_revision_table)
     
-vdm.sqlalchemy.modify_base_object_mapper(ResourceGroup, Revision, State)
+vdm.sqlalchemy.modify_base_object_mapper(ResourceGroup, core.Revision, core.State)
 ResourceGroupRevision = vdm.sqlalchemy.create_object_version(
-    mapper, ResourceGroup, resource_group_revision_table)
+    meta.mapper, ResourceGroup, resource_group_revision_table)
 
 ResourceGroupRevision.related_packages = lambda self: [self.continuity.package]
 ResourceRevision.related_packages = lambda self: [self.continuity.resouce_group.package]
 
-import vdm.sqlalchemy.stateful
-# TODO: move this into vdm
-def add_stateful_m21(object_to_alter, m21_property_name,
-        underlying_m21_attrname, identifier, **kwargs):
-    from sqlalchemy.orm import object_session
-    def _f(obj_to_delete):
-        sess = object_session(obj_to_delete)
-        if sess: # for tests at least must support obj not being sqlalchemy
-            sess.expunge(obj_to_delete)
-
-    active_list = vdm.sqlalchemy.stateful.DeferredProperty(
-            underlying_m21_attrname,
-            vdm.sqlalchemy.stateful.StatefulList,
-            # these args are passed to StatefulList
-            # identifier if url (could use id but have issue with None)
-            identifier=identifier,
-            unneeded_deleter=_f,
-            base_modifier=lambda x: x.get_as_of()
-            )
-    setattr(object_to_alter, m21_property_name, active_list)
-
 def resource_identifier(obj):
     return obj.id
 
-
-add_stateful_m21(Package, 'resource_groups', 'resource_groups_all',
-                 resource_identifier)
-add_stateful_m21(ResourceGroup, 'resources', 'resources_all',
-                 resource_identifier)
-
-
-
 class DictProxy(object):
 
     def __init__(self, target_key, target_dict, data_type=unicode):
