@@ -1,42 +1,45 @@
-import sqlalchemy
 import vdm.sqlalchemy
+from sqlalchemy.orm import relation
+from sqlalchemy import types, Column, Table, ForeignKey, and_, UniqueConstraint
 
-from types import make_uuid
-from meta import *
-from domain_object import DomainObject
-from package import Package
-from core import *
+import package as _package
+import extension as _extension
+import core
+import meta
+import types as _types
+import domain_object
 import vocabulary
 import activity
-import ckan
+import ckan  # this import is needed
+import ckan.lib.dictization
 
 __all__ = ['tag_table', 'package_tag_table', 'Tag', 'PackageTag',
-           'PackageTagRevision', 'package_tag_revision_table',
+           'package_tag_revision_table',
            'MAX_TAG_LENGTH', 'MIN_TAG_LENGTH']
 
 MAX_TAG_LENGTH = 100
 MIN_TAG_LENGTH = 2
 
-tag_table = Table('tag', metadata,
-        Column('id', types.UnicodeText, primary_key=True, default=make_uuid),
+tag_table = Table('tag', meta.metadata,
+        Column('id', types.UnicodeText, primary_key=True, default=_types.make_uuid),
         Column('name', types.Unicode(MAX_TAG_LENGTH), nullable=False),
         Column('vocabulary_id',
             types.Unicode(vocabulary.VOCABULARY_NAME_MAX_LENGTH),
             ForeignKey('vocabulary.id')),
-        sqlalchemy.UniqueConstraint('name', 'vocabulary_id')
+        UniqueConstraint('name', 'vocabulary_id')
 )
 
-package_tag_table = Table('package_tag', metadata,
-        Column('id', types.UnicodeText, primary_key=True, default=make_uuid),
+package_tag_table = Table('package_tag', meta.metadata,
+        Column('id', types.UnicodeText, primary_key=True, default=_types.make_uuid),
         Column('package_id', types.UnicodeText, ForeignKey('package.id')),
         Column('tag_id', types.UnicodeText, ForeignKey('tag.id')),
         )
 
 vdm.sqlalchemy.make_table_stateful(package_tag_table)
 # TODO: this has a composite primary key ...
-package_tag_revision_table = make_revisioned_table(package_tag_table)
+package_tag_revision_table = core.make_revisioned_table(package_tag_table)
 
-class Tag(DomainObject):
+class Tag(domain_object.DomainObject):
     def __init__(self, name='', vocabulary_id=None):
         self.name = name
         self.vocabulary_id = vocabulary_id
@@ -47,38 +50,47 @@ class Tag(DomainObject):
 
     @classmethod
     def by_id(cls, tag_id, autoflush=True):
-        """Return the tag object with the given id, or None if there is no
-        tag with that id.
+        '''Return the tag with the given id, or None.
 
-        Arguments:
-        tag_id -- The id of the tag to return.
+        :param tag_id: the id of the tag to return
+        :type tag_id: string
 
-        """
-        query = Session.query(Tag).filter(Tag.id==tag_id)
+        :returns: the tag with the given id, or None if there is no tag with
+            that id
+        :rtype: ckan.model.tag.Tag
+
+        '''
+        query = meta.Session.query(Tag).filter(Tag.id==tag_id)
         query = query.autoflush(autoflush)
         tag = query.first()
         return tag
 
     @classmethod
     def by_name(cls, name, vocab=None, autoflush=True):
-        """Return the tag object with the given name, or None if there is no
-        tag with that name.
+        '''Return the tag with the given name, or None.
 
         By default only free tags (tags which do not belong to any vocabulary)
-        are returned. If the optional argument vocab is given then only tags
-        from that vocabulary are returned, or None if there is no tag with that
-        name in that vocabulary.
+        are returned.
 
-        Arguments:
-        name -- The name of the tag to return.
-        vocab -- A Vocabulary object for the vocabulary to look in (optional).
+        If the optional argument ``vocab`` is given then only tags from that
+        vocabulary are returned, or ``None`` if there is no tag with that name
+        in that vocabulary.
 
-        """
+        :param name: the name of the tag to return
+        :type name: string
+        :param vocab: the vocabulary to look in (optional, default: None)
+        :type vocab: ckan.model.vocabulary.Vocabulary
+
+        :returns: the tag object with the given id or name, or None if there is
+            no tag with that id or name
+        :rtype: ckan.model.tag.Tag
+
+        '''
         if vocab:
-            query = Session.query(Tag).filter(Tag.name==name).filter(
+            query = meta.Session.query(Tag).filter(Tag.name==name).filter(
                 Tag.vocabulary_id==vocab.id)
         else:
-            query = Session.query(Tag).filter(Tag.name==name).filter(
+            query = meta.Session.query(Tag).filter(Tag.name==name).filter(
                 Tag.vocabulary_id==None)
         query = query.autoflush(autoflush)
         tag = query.first()
@@ -86,20 +98,27 @@ class Tag(DomainObject):
 
     @classmethod
     def get(cls, tag_id_or_name, vocab_id_or_name=None):
-        """Return the tag object with the given id or name, or None if there is
-        no tag with that id or name.
+        '''Return the tag with the given id or name, or None.
 
         By default only free tags (tags which do not belong to any vocabulary)
-        are returned. If the optional argument vocab_id_or_name is given then
-        only tags that belong to that vocabulary will be returned, and None
-        will be returned if there is no vocabulary with that vocabulary id or
-        name or if there is no tag with that tag id or name in that vocabulary.
+        are returned.
 
-        Arguments:
-        tag_id_or_name -- The id or name of the tag to return.
-        vocab_id_or_name -- The id or name of the vocabulary to look in.
+        If the optional argument ``vocab_id_or_name`` is given then only tags
+        that belong to that vocabulary will be returned, and ``None`` will be
+        returned if there is no vocabulary with that vocabulary id or name or
+        if there is no tag with that tag id or name in that vocabulary.
 
-        """
+        :param tag_id_or_name: the id or name of the tag to return
+        :type tag_id_or_name: string
+        :param vocab_id_or_name: the id or name of the vocabulary to look for
+            the tag in
+        :type vocab_id_or_name: string
+
+        :returns: the tag object with the given id or name, or None if there is
+            no tag with that id or name
+        :rtype: ckan.model.tag.Tag
+
+        '''
         # First try to get the tag by ID.
         tag = Tag.by_id(tag_id_or_name)
         if tag:
@@ -120,21 +139,30 @@ class Tag(DomainObject):
 
     @classmethod
     def search_by_name(cls, search_term, vocab_id_or_name=None):
-        """Return all tags that match the given search term.
+        '''Return all tags whose names contain a given string.
 
         By default only free tags (tags which do not belong to any vocabulary)
-        are returned. If the optional argument vocab_id_or_name is given then
-        only tags from that vocabulary are returned.
+        are returned. If the optional argument ``vocab_id_or_name`` is given
+        then only tags from that vocabulary are returned.
 
-        """
+        :param search_term: the string to search for in the tag names
+        :type search_term: string
+        :param vocab_id_or_name: the id or name of the vocabulary to look in
+            (optional, default: None)
+        :type vocab_id_or_name: string
+
+        :returns: a list of tags that match the search term
+        :rtype: list of ckan.model.tag.Tag objects
+
+        '''
         if vocab_id_or_name:
             vocab = vocabulary.Vocabulary.get(vocab_id_or_name)
             if vocab is None:
                 # The user specified an invalid vocab.
                 return None
-            query = Session.query(Tag).filter(Tag.vocabulary_id==vocab.id)
+            query = meta.Session.query(Tag).filter(Tag.vocabulary_id==vocab.id)
         else:
-            query = Session.query(Tag)
+            query = meta.Session.query(Tag)
         search_term = search_term.strip().lower()
         query = query.filter(Tag.name.contains(search_term))
         query = query.distinct().join(Tag.package_tags)
@@ -142,42 +170,49 @@ class Tag(DomainObject):
 
     @classmethod
     def all(cls, vocab_id_or_name=None):
-        """Return all tags that are currently applied to a package.
+        '''Return all tags that are currently applied to any dataset.
 
         By default only free tags (tags which do not belong to any vocabulary)
-        are returned. If the optional argument vocab_id_or_name is given then
-        only tags from that vocabulary are returned.
+        are returned. If the optional argument ``vocab_id_or_name`` is given
+        then only tags from that vocabulary are returned.
 
-        """
+        :param vocab_id_or_name: the id or name of the vocabulary to look in
+            (optional, default: None)
+        :type vocab_id_or_name: string
+
+        :returns: a list of all tags that are currently applied to any dataset
+        :rtype: list of ckan.model.tag.Tag objects
+
+        '''
         if vocab_id_or_name:
             vocab = vocabulary.Vocabulary.get(vocab_id_or_name)
             if vocab is None:
                 # The user specified an invalid vocab.
                 raise ckan.logic.NotFound("could not find vocabulary '%s'"
                         % vocab_id_or_name)
-            query = Session.query(Tag).filter(Tag.vocabulary_id==vocab.id)
+            query = meta.Session.query(Tag).filter(Tag.vocabulary_id==vocab.id)
         else:
-            query = Session.query(Tag).filter(Tag.vocabulary_id == None)
-            query = query.distinct().join(PackageTagRevision)
-            query = query.filter(sqlalchemy.and_(
-                PackageTagRevision.state == 'active',
-                PackageTagRevision.current == True))
+            query = meta.Session.query(Tag).filter(Tag.vocabulary_id == None)
+            query = query.distinct().join(_package.PackageTagRevision)
+            query = query.filter(and_(
+                _package.PackageTagRevision.state == 'active',
+                _package.PackageTagRevision.current == True))
         return query
 
     @property
     def packages(self):
-        """Return a list of all packages currently tagged with this tag.
+        '''Return a list of all packages that have this tag, sorted by name.
 
-        The list is sorted by package name.
+        :rtype: list of ckan.model.package.Package objects
 
-        """
-        q = Session.query(Package)
-        q = q.join(PackageTagRevision)
-        q = q.filter(PackageTagRevision.tag_id == self.id)
-        q = q.filter(sqlalchemy.and_(
-            PackageTagRevision.state == 'active',
-            PackageTagRevision.current == True))
-        q = q.order_by(Package.name)
+        '''
+        q = meta.Session.query(_package.Package)
+        q = q.join(_package.PackageTagRevision)
+        q = q.filter(_package.PackageTagRevision.tag_id == self.id)
+        q = q.filter(and_(
+            _package.PackageTagRevision.state == 'active',
+            _package.PackageTagRevision.current == True))
+        q = q.order_by(_package.Package.name)
         packages = q.all()
         return packages
 
@@ -186,7 +221,7 @@ class Tag(DomainObject):
 
 class PackageTag(vdm.sqlalchemy.RevisionedObjectMixin,
         vdm.sqlalchemy.StatefulObjectMixin,
-        DomainObject):
+        domain_object.DomainObject):
     def __init__(self, package=None, tag=None, state=None, **kwargs):
         self.package = package
         self.tag = tag
@@ -213,9 +248,8 @@ class PackageTag(vdm.sqlalchemy.RevisionedObjectMixin,
             return None
 
         # Return an 'added tag' or 'removed tag' activity.
-        import ckan.lib.dictization
-        import ckan.model
-        c = {'model': ckan.model}
+        import ckan.model as model
+        c = {'model': model}
         d = {'tag': ckan.lib.dictization.table_dictize(self.tag, c),
             'package': ckan.lib.dictization.table_dictize(self.package, c)}
         return activity.ActivityDetail(
@@ -228,27 +262,38 @@ class PackageTag(vdm.sqlalchemy.RevisionedObjectMixin,
     @classmethod
     def by_name(self, package_name, tag_name, vocab_id_or_name=None,
             autoflush=True):
-        """Return the one PackageTag for the given package and tag names, or
-        None if there is no PackageTag for that package and tag.
+        '''Return the PackageTag for the given package and tag names, or None.
 
         By default only PackageTags for free tags (tags which do not belong to
-        any vocabulary) are returned. If the optional argument vocab_id_or_name
-        is given then only PackageTags for tags from that vocabulary are
-        returned.
+        any vocabulary) are returned. If the optional argument
+        ``vocab_id_or_name`` is given then only PackageTags for tags from that
+        vocabulary are returned.
 
-        """
+        :param package_name: the name of the package to look for
+        :type package_name: string
+        :param tag_name: the name of the tag to look for
+        :type tag_name: string
+        :param vocab_id_or_name: the id or name of the vocabulary to look for
+            the tag in
+        :type vocab_id_or_name: string
+
+        :returns: the PackageTag for the given package and tag names, or None
+            if there is no PackageTag for those package and tag names
+        :rtype: ckan.model.tag.PackageTag
+
+        '''
         if vocab_id_or_name:
             vocab = vocabulary.Vocabulary.get(vocab_id_or_name)
             if vocab is None:
                 # The user specified an invalid vocab.
                 return None
-            query = (Session.query(PackageTag, Tag, Package)
+            query = (meta.Session.query(PackageTag, Tag, _package.Package)
                     .filter(Tag.vocabulary_id == vocab.id)
-                    .filter(Package.name==package_name)
+                    .filter(_package.Package.name==package_name)
                     .filter(Tag.name==tag_name))
         else:
-            query = (Session.query(PackageTag)
-                    .filter(Package.name==package_name)
+            query = (meta.Session.query(PackageTag)
+                    .filter(_package.Package.name==package_name)
                     .filter(Tag.name==tag_name))
         query = query.autoflush(autoflush)
         return query.one()[0]
@@ -256,7 +301,7 @@ class PackageTag(vdm.sqlalchemy.RevisionedObjectMixin,
     def related_packages(self):
         return [self.package]
 
-mapper(Tag, tag_table, properties={
+meta.mapper(Tag, tag_table, properties={
     'package_tags': relation(PackageTag, backref='tag',
         cascade='all, delete, delete-orphan',
         ),
@@ -266,21 +311,13 @@ mapper(Tag, tag_table, properties={
     order_by=tag_table.c.name,
     )
 
-mapper(PackageTag, package_tag_table, properties={
-    'pkg':relation(Package, backref='package_tag_all',
+meta.mapper(PackageTag, package_tag_table, properties={
+    'pkg':relation(_package.Package, backref='package_tag_all',
         cascade='none',
         )
     },
     order_by=package_tag_table.c.id,
     extension=[vdm.sqlalchemy.Revisioner(package_tag_revision_table),
-               extension.PluginMapperExtension(),
+               _extension.PluginMapperExtension(),
                ],
     )
-
-from package_mapping import *
-
-vdm.sqlalchemy.modify_base_object_mapper(PackageTag, Revision, State)
-PackageTagRevision = vdm.sqlalchemy.create_object_version(mapper, PackageTag,
-        package_tag_revision_table)
-
-PackageTagRevision.related_packages = lambda self: [self.continuity.package]
