@@ -151,10 +151,23 @@ CKAN.Utils = CKAN.Utils || {};
 jQuery.fn.truncate = function (max, suffix) {
   return this.each(function () {
     var element = jQuery(this),
-        cached  = element.text(),
-        length  = max || element.data('truncate') || 30,
-        text    = cached.slice(0, length),
-        expand  = jQuery('<a href="#" />').text(suffix || '»');
+        isTruncated = element.hasClass('truncated'),
+        cached, length, text, expand;
+
+    if (isTruncated) {
+      element.html(element.data('truncate:' + (max === 'expand' ? 'original' : 'truncated')));
+      return;
+    }
+
+    cached  = element.text();
+    length  = max || element.data('truncate') || 30;
+    text    = cached.slice(0, length);
+    expand  = jQuery('<a href="#" />').text(suffix || '»');
+
+    // Bail early if there is nothing to truncate.
+    if (cached.length < length) {
+      return;
+    }
 
     // Try to truncate to nearest full word.
     while ((/\S/).test(text[text.length - 1])) {
@@ -162,12 +175,15 @@ jQuery.fn.truncate = function (max, suffix) {
     }
 
     element.html(jQuery.trim(text));
-
     expand.appendTo(element.append(' '));
     expand.click(function (event) {
       event.preventDefault();
-      element.text(cached);
+      element.html(cached);
     });
+
+    element.addClass('truncated');
+    element.data('truncate:original',  cached);
+    element.data('truncate:truncated', element.html());
   });
 };
 
@@ -475,7 +491,7 @@ CKAN.View.ResourceEditor = Backbone.View.extend({
 CKAN.View.Resource = Backbone.View.extend({
   initialize: function() {
     this.el = $(this.el);
-    _.bindAll(this,'updateName','updateIcon','name','askToDelete','openMyPanel','setErrors','setupDynamicExtras','addDynamicExtra', 'onDatastoreEnabledChange');
+    _.bindAll(this,'updateName','updateIcon','name','askToDelete','openMyPanel','setErrors','setupDynamicExtras','addDynamicExtra' );
     this.render();
   },
   render: function() {
@@ -510,12 +526,8 @@ CKAN.View.Resource = Backbone.View.extend({
     // Hook to open panel link
     this.li.find('.resource-open-my-panel').click(this.openMyPanel);
     this.table.find('.js-resource-edit-delete').click(this.askToDelete);
-    this.table.find('.js-datastore-enabled-checkbox').change(this.onDatastoreEnabledChange);
     // Hook to markdown editor
     CKAN.Utils.setupMarkdownEditor(this.table.find('.markdown-editor'));
-    if (resource_object.resource.webstore_url) {
-      this.table.find('.js-datastore-enabled-checkbox').prop('checked', true);
-    }
 
     // Set initial state
     this.updateName();
@@ -713,12 +725,6 @@ CKAN.View.Resource = Backbone.View.extend({
   removeFromDom: function() {
     this.li.remove();
     this.table.remove();
-  },
-  onDatastoreEnabledChange: function(e) {
-    var isChecked = this.table.find('.js-datastore-enabled-checkbox').prop('checked');
-    var webstore_url = isChecked ? 'enabled' : null;
-    this.model.set({webstore_url: webstore_url});
-    this.table.find('.js-datastore-enabled-text').val(webstore_url);
   }
 });
 
@@ -851,7 +857,6 @@ CKAN.View.ResourceAddUpload = Backbone.View.extend({
             , hash: data._checksum
             , cache_url: data._location
             , cache_url_updated: lastmod
-            , webstore_url: data._location
           }
           , {
             error: function(model, error) {
@@ -918,7 +923,6 @@ CKAN.View.ResourceAddUrl = Backbone.View.extend({
              size: data.size,
              mimetype: data.mimetype,
              last_modified: data.last_modified,
-             webstore_url: 'enabled',
              url_error: (data.url_errors || [""])[0]
            });
            self.collection.add(newResource);
@@ -928,9 +932,6 @@ CKAN.View.ResourceAddUrl = Backbone.View.extend({
      }
      else {
        newResource.set({url: urlVal, resource_type: this.options.mode});
-       if (newResource.get('resource_type')=='file') {
-         newResource.set({webstore_url: 'enabled'});
-       }
        this.collection.add(newResource);
        this.resetForm();
      }
@@ -1018,7 +1019,7 @@ CKAN.Utils = function($, my) {
 
         input_box.attr('name', new_name);
         input_box.attr('id', new_name);
-        
+
         var $new = $('<div class="ckan-dataset-to-add"><p></p></div>');
         $new.append($('<input type="hidden" />').attr('name', old_name).val(ui.item.value));
         $new.append('<i class="icon-plus-sign"></i> ');
@@ -1159,13 +1160,30 @@ CKAN.Utils = function($, my) {
 
 
   my.relatedSetup = function(form) {
+    $('[rel=popover]').popover();
+
     function addAlert(msg) {
       $('<div class="alert alert-error" />').html(msg).hide().prependTo(form).fadeIn();
     }
 
+    function relatedRequest(action, method, data) {
+      return $.ajax({
+        type: method,
+        dataType: 'json',
+        contentType: 'application/json',
+        url: CKAN.SITE_URL + '/api/3/action/related_' + action,
+        data: data ? JSON.stringify(data) : undefined,
+        error: function(err, txt, w) {
+          // This needs to be far more informative.
+          addAlert('<strong>Error:</strong> Unable to ' + action + ' related item');
+        }
+      });
+    }
+
     // Center thumbnails vertically.
-    $('.related-items').each(function () {
-      var item = $(this);
+    var relatedItems = $('.related-items');
+    relatedItems.find('li').each(function () {
+      var item = $(this), description = item.find('.description');
 
       function vertiallyAlign() {
         var img = $(this),
@@ -1179,7 +1197,47 @@ CKAN.Utils = function($, my) {
       }
 
       item.find('img').load(vertiallyAlign);
-      item.find('.description').truncate();
+      description.data('height', description.height()).truncate();
+    });
+
+    relatedItems.on('mouseenter mouseleave', '.description.truncated', function (event) {
+      var isEnter = event.type === 'mouseenter'
+          description = $(this)
+          timer = description.data('hover-intent');
+
+      function update() {
+        var parent = description.parents('li:first'),
+            difference = description.data('height') - description.height();
+
+        description.truncate(isEnter ? 'expand' : 'collapse');
+        parent.toggleClass('expanded-description', isEnter);
+
+        // Adjust the bottom margin of the item relative to it's current value
+        // to allow the description to expand without breaking the grid.
+        parent.css('margin-bottom', isEnter ? '-=' + difference + 'px' : '');
+        description.removeData('hover-intent');
+      }
+
+      if (!isEnter && timer) {
+        // User has moused out in the time set so cancel the action.
+        description.removeData('hover-intent');
+        return clearTimeout(timer);
+      } else if (!isEnter && !timer) {
+        update();
+      } else {
+        // Delay the hover action slightly to wait to see if the user mouses
+        // out again. This prevents unwanted actions.
+        description.data('hover-intent', setTimeout(update, 200));
+      }
+    });
+
+    // Add a handler for the delete buttons.
+    relatedItems.on('click', '[data-action=delete]', function (event) {
+      var id = $(this).data('relatedId');
+      relatedRequest('delete', 'POST', {id: id}).done(function () {
+        $('#related-item-' + id).remove();
+      });
+      event.preventDefault();
     });
 
     $(form).submit(function (event) {
@@ -1204,18 +1262,10 @@ CKAN.Utils = function($, my) {
         return;
       }
 
-      $.ajax({
-        type: this.method,
-        url: CKAN.SITE_URL + '/api/3/action/related_create',
-        data: JSON.stringify(data),
-        success: function (data) {
-          window.location.reload();
-        },
-        error: function(err, txt, w) {
-          // This needs to be far more informative.
-          addAlert('<strong>Error:</strong> Unable to add related item');
-        }
-      }); 
+      relatedRequest('create', this.method, data).done(function () {
+        // TODO: Insert item dynamically.
+        window.location.reload();
+      });
     });
   };
 
@@ -1355,6 +1405,71 @@ CKAN.Utils = function($, my) {
     });
     return count;
   };
+
+  function followButtonClicked(event) {
+    var button = event.currentTarget;
+    if (button.id === 'user_follow_button') {
+        var object_type = 'user';
+    } else if (button.id === 'dataset_follow_button') {
+        var object_type = 'dataset';
+    }
+    else {
+        // This shouldn't happen.
+        return;
+    }
+	var object_id = button.getAttribute('data-obj-id');
+    if (button.getAttribute('data-state') === "follow") {
+        if (object_type == 'user') {
+            var url = '/api/action/follow_user';
+        } else if (object_type == 'dataset') {
+            var url = '/api/action/follow_dataset';
+        } else {
+            // This shouldn't happen.
+            return;
+        }
+        var data = JSON.stringify({
+          id: object_id
+        });
+        var nextState = 'unfollow';
+        var nextString = CKAN.Strings.unfollow;
+    } else if (button.getAttribute('data-state') === "unfollow") {
+        if (object_type == 'user') {
+            var url = '/api/action/unfollow_user';
+        } else if (object_type == 'dataset') {
+            var url = '/api/action/unfollow_dataset';
+        } else {
+            // This shouldn't happen.
+            return;
+        }
+        var data = JSON.stringify({
+          id: object_id
+        });
+        var nextState = 'follow';
+        var nextString = CKAN.Strings.follow;
+    }
+    else {
+        // This shouldn't happen.
+        return;
+    }
+    $.ajax({
+      contentType: 'application/json',
+      url: url,
+      data: data,
+      dataType: 'json',
+      processData: false,
+      type: 'POST',
+      success: function(data) {
+        button.setAttribute('data-state', nextState);
+        button.innerHTML = nextString;
+      }
+    });
+  };
+
+  // This only needs to happen on dataset pages, but it doesn't seem to do
+  // any harm to call it anyway.
+  $('#user_follow_button').on('click', followButtonClicked);
+  $('#dataset_follow_button').on('click', followButtonClicked);
+
   return my;
 }(jQuery, CKAN.Utils || {});
 
@@ -1378,7 +1493,13 @@ CKAN.DataPreview = function ($, my) {
     my.$dialog.html('<h4>Loading ... <img src="http://assets.okfn.org/images/icons/ajaxload-circle.gif" class="loading-spinner" /></h4>');
 
     // Restore the Dataset from the given reclineState.
-    var dataset = recline.Model.Dataset.restore(reclineState);
+    var datasetInfo = _.extend({
+        url: reclineState.url,
+        backend: reclineState.backend
+      },
+      reclineState.dataset
+    );
+    var dataset = new recline.Model.Dataset(datasetInfo);
 
     // Only create the view defined in reclineState.currentView.
     // TODO: tidy this up.
@@ -1387,7 +1508,7 @@ CKAN.DataPreview = function ($, my) {
       views = [ {
         id: 'grid',
         label: 'Grid',
-        view: new recline.View.Grid({
+        view: new recline.View.SlickGrid({
           model: dataset,
           state: reclineState['view-grid']
         })
@@ -1413,14 +1534,13 @@ CKAN.DataPreview = function ($, my) {
     }
 
     // Finally, construct the DataExplorer.  Again, passing in the reclineState.
-    var dataExplorer = new recline.View.DataExplorer({
+    var dataExplorer = new recline.View.MultiView({
       el: my.$dialog,
       model: dataset,
       state: reclineState,
       views: views
     });
 
-    Backbone.history.start();
   };
 
   // **Public: Creates a link to the embeddable page.
@@ -1455,12 +1575,20 @@ CKAN.DataPreview = function ($, my) {
   my.loadPreviewDialog = function(resourceData) {
     my.$dialog.html('<h4>Loading ... <img src="http://assets.okfn.org/images/icons/ajaxload-circle.gif" class="loading-spinner" /></h4>');
 
+    function showError(msg){
+      msg = msg || CKAN.Strings.errorLoadingPreview;
+      return $('#ckanext-datapreview')
+        .append('<div></div>')
+        .addClass('alert alert-error fade in')
+        .html(msg);
+    }
+
     function initializeDataExplorer(dataset) {
       var views = [
         {
           id: 'grid',
           label: 'Grid',
-          view: new recline.View.Grid({
+          view: new recline.View.SlickGrid({
             model: dataset
           })
         },
@@ -1479,7 +1607,8 @@ CKAN.DataPreview = function ($, my) {
           })
         }
       ];
-      var dataExplorer = new recline.View.DataExplorer({
+
+      var dataExplorer = new recline.View.MultiView({
         el: my.$dialog,
         model: dataset,
         views: views,
@@ -1488,6 +1617,10 @@ CKAN.DataPreview = function ($, my) {
         }
       });
 
+      // Hide the fields control by default
+      // (This should be done in recline!)
+      $('.menu-right a[data-action="fields"]').click();
+
       // -----------------------------
       // Setup the Embed modal dialog.
       // -----------------------------
@@ -1539,19 +1672,16 @@ CKAN.DataPreview = function ($, my) {
       // Finally, since we have a DataExplorer, we can show the embed button.
       $('.preview-header .btn').show();
 
-      // will have to refactor if this can get called multiple times
-      Backbone.history.start();
     }
 
     // 4 situations
-    // a) have a webstore_url
-    // b) csv or xls (but not webstore)
+    // a) something was posted to the datastore - need to check for this
+    // b) csv or xls (but not datastore)
     // c) can be treated as plain text
     // d) none of the above but worth iframing (assumption is
     // that if we got here (i.e. preview shown) worth doing
     // something ...)
     resourceData.formatNormalized = my.normalizeFormat(resourceData.format);
-
     resourceData.url  = my.normalizeUrl(resourceData.url);
     if (resourceData.formatNormalized === '') {
       var tmp = resourceData.url.split('/');
@@ -1564,17 +1694,46 @@ CKAN.DataPreview = function ($, my) {
       }
     }
 
-    if (resourceData.webstore_url) {
-      resourceData.elasticsearch_url = '/api/data/' + resourceData.id;
-      var dataset = new recline.Model.Dataset(resourceData, 'elasticsearch');
-      initializeDataExplorer(dataset);
+    // Set recline CKAN backend API endpoint to right location (so it can locate
+    // CKAN DataStore)
+    recline.Backend.Ckan.API_ENDPOINT = CKAN.SITE_URL + '/api';
+
+    if (resourceData.datastore_active) {
+      resourceData.backend =  'ckan';
+      var dataset = new recline.Model.Dataset(resourceData);
+      var errorMsg = CKAN.Strings.errorLoadingPreview + ': ' + CKAN.Strings.errorDataStore;
+      dataset.fetch()
+        .done(function(dataset){
+            initializeDataExplorer(dataset);
+        })
+        .fail(function(error){
+          if (error.message) errorMsg += ' (' + error.message + ')';
+          showError(errorMsg);
+        });
+
     }
-    else if (resourceData.formatNormalized in {'csv': '', 'xls': ''}) {
+    else if (resourceData.formatNormalized in {'csv': '', 'xls': '', 'tsv':''}) {
       // set format as this is used by Recline in setting format for DataProxy
       resourceData.format = resourceData.formatNormalized;
-      var dataset = new recline.Model.Dataset(resourceData, 'dataproxy');
-      initializeDataExplorer(dataset);
-      $('.recline-query-editor .text-query').hide();
+      resourceData.backend = 'dataproxy';
+      var dataset = new recline.Model.Dataset(resourceData);
+      var errorMsg = CKAN.Strings.errorLoadingPreview + ': ' +CKAN.Strings.errorDataProxy;
+      dataset.fetch()
+        .done(function(dataset){
+
+          dataset.bind('query:fail', function(error) {
+            $('#ckanext-datapreview .data-view-container').hide();
+            $('#ckanext-datapreview .header').hide();
+            $('.preview-header .btn').hide();
+          });
+
+          initializeDataExplorer(dataset);
+          $('.recline-query-editor .text-query').hide();
+        })
+        .fail(function(error){
+          if (error.message) errorMsg += ' (' + error.message + ')';
+          showError(errorMsg);
+        });
     }
     else if (resourceData.formatNormalized in {
         'rdf+xml': '',
