@@ -29,6 +29,8 @@ from ckan.lib.create_test_data import CreateTestData
 from ckan.lib import search
 from ckan.lib.helpers import _flash, url_for
 from ckan.lib.helpers import json
+from ckan.logic import get_action
+from ckan.logic.action import get_domain_object
 import ckan.model as model
 from ckan import ckan_nose_plugin
 
@@ -317,6 +319,9 @@ def is_search_supported():
     is_supported_db = not model.engine_is_sqlite()
     return is_supported_db
 
+def are_foreign_keys_supported():
+    return not model.engine_is_sqlite()
+
 def is_regex_supported():
     is_supported_db = not model.engine_is_sqlite()
     return is_supported_db
@@ -345,10 +350,49 @@ def clear_flash(res=None):
 try:
     from nose.tools import assert_in, assert_not_in
 except ImportError:
-    def assert_in(a, b):
-        assert a in b, '%r was not in %r' % (a, b)
-    def assert_not_in(a, b):
-        assert a not in b, '%r was in %r' % (a, b)        
+    def assert_in(a, b, msg=None):
+        assert a in b, msg or '%r was not in %r' % (a, b)
+    def assert_not_in(a, b, msg=None):
+        assert a not in b, msg or '%r was in %r' % (a, b)        
+
+class TestRoles:
+    @classmethod
+    def get_roles(cls, domain_object_ref, user_ref=None, authgroup_ref=None,
+                  prettify=True):
+        data_dict = {'domain_object': domain_object_ref}
+        if user_ref:
+            data_dict['user'] = user_ref
+        if authgroup_ref:
+            data_dict['authorization_group'] = authgroup_ref
+        role_dicts = get_action('roles_show') \
+                     ({'model': model, 'session': model.Session}, \
+                      data_dict)['roles']
+        if prettify:
+            role_dicts = cls.prettify_role_dicts(role_dicts)
+        return role_dicts
+
+    @classmethod
+    def prettify_role_dicts(cls, role_dicts, one_per_line=True):
+        '''Replace ids with names'''
+        pretty_roles = []
+        for role_dict in role_dicts:
+            pretty_role = {}
+            for key, value in role_dict.items():
+                if key.endswith('_id') and value and key != 'user_object_role_id':
+                    pretty_key = key[:key.find('_id')]
+                    domain_object = get_domain_object(model, value)
+                    pretty_value = domain_object.name
+                    pretty_role[pretty_key] = pretty_value
+                else:
+                    pretty_role[key] = value
+            if one_per_line:
+                pretty_role = '"%s" is "%s" on "%s"' % (
+                    pretty_role.get('user') or pretty_role.get('authorized_group'),
+                    pretty_role['role'],
+                    pretty_role.get('package') or pretty_role.get('group') or pretty_role.get('authorization_group') or pretty_role.get('context'))
+            pretty_roles.append(pretty_role)
+        return pretty_roles
+    
 
 class StatusCodes:
     STATUS_200_OK = 200
