@@ -2,11 +2,12 @@ import re
 
 from nose.tools import assert_equal
 
+from ckan.tests import setup_test_search_index
 from ckan.plugins import SingletonPlugin, implements, IGroupController
 from ckan import plugins
 import ckan.model as model
 from ckan.lib.create_test_data import CreateTestData
-from ckan.logic import check_access, NotAuthorized
+from ckan.logic import check_access, NotAuthorized, get_action
 
 from pylons import config
 
@@ -15,6 +16,7 @@ from ckan.tests import setup_test_search_index
 from base import FunctionalTestCase
 from ckan.tests import search_related, is_search_supported
 
+
 class MockGroupControllerPlugin(SingletonPlugin):
     implements(IGroupController)
 
@@ -44,6 +46,7 @@ class MockGroupControllerPlugin(SingletonPlugin):
         self.calls['before_view'] += 1
         return data_dict
 
+
 class TestGroup(FunctionalTestCase):
 
     @classmethod
@@ -55,6 +58,114 @@ class TestGroup(FunctionalTestCase):
     def teardown_class(self):
         model.repo.rebuild_db()
 
+    def test_atom_feed_page_zero(self):
+        group_name = 'deletetest'
+        CreateTestData.create_groups([{'name': group_name,
+                                       'packages': []}],
+                                     admin_user_name='russianfan')
+
+        offset = url_for(controller='feed', action='group',
+                         id=group_name)
+        offset = offset + '?page=0'
+        res = self.app.get(offset)
+        assert '<feed' in res, res
+        assert 'xmlns="http://www.w3.org/2005/Atom"' in res, res
+        assert '</feed>' in res, res
+
+    def test_children(self):
+        if model.engine_is_sqlite() :
+            from nose import SkipTest
+            raise SkipTest("Can't use CTE for sqlite")
+
+        group_name = 'deletetest'
+        CreateTestData.create_groups([{'name': group_name,
+                                       'packages': []},
+                                       {'name': "parent_group",
+                                       'packages': []}],
+                                     admin_user_name='russianfan')
+
+        parent = model.Group.by_name("parent_group")
+        group = model.Group.by_name(group_name)
+
+        rev = model.repo.new_revision()
+        rev.author = "none"
+
+        member = model.Member(group_id=parent.id, table_id=group.id,
+                              table_name='group', capacity='member')
+        model.Session.add(member)
+        model.repo.commit_and_remove()
+
+        offset = url_for(controller='group', action='edit', id=group_name)
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
+        main_res = self.main_div(res)
+        assert 'Edit: %s' % group.title in main_res, main_res
+        assert 'value="active" selected' in main_res, main_res
+
+        parent = model.Group.by_name("parent_group")
+        assert_equal(len(parent.get_children_groups()), 1)
+
+        # delete
+        form = res.forms['group-edit']
+        form['state'] = 'deleted'
+        res = form.submit('save', status=302,
+                          extra_environ={'REMOTE_USER': 'russianfan'})
+
+        group = model.Group.by_name(group_name)
+        assert_equal(group.state, 'deleted')
+
+        parent = model.Group.by_name("parent_group")
+        assert_equal(len(parent.get_children_groups()), 0)
+
+    def test_sorting(self):
+        model.repo.rebuild_db()
+
+        pkg1 = model.Package(name="pkg1")
+        pkg2 = model.Package(name="pkg2")
+        model.Session.add(pkg1)
+        model.Session.add(pkg2)
+
+        CreateTestData.create_groups([{'name': "alpha", 'packages': []},
+                                       {'name': "beta",
+                                        'packages': ["pkg1", "pkg2"]},
+                                       {'name': "delta",
+                                        'packages': ["pkg1"]},
+                                       {'name': "gamma", 'packages': []}],
+                                     admin_user_name='russianfan')
+
+        context = {'model': model, 'session': model.Session,
+                   'user': 'russianfan', 'for_view': True,
+                   'with_private': False}
+        data_dict = {'all_fields': True}
+        results = get_action('group_list')(context, data_dict)
+        assert results[0]['name'] == u'alpha', results[0]['name']
+        assert results[-1]['name'] == u'gamma', results[-1]['name']
+
+        # Test name reverse
+        data_dict = {'all_fields': True, 'sort': 'name desc'}
+        results = get_action('group_list')(context, data_dict)
+        assert results[0]['name'] == u'gamma', results[0]['name']
+        assert results[-1]['name'] == u'alpha', results[-1]['name']
+
+        # Test packages reversed
+        data_dict = {'all_fields': True, 'sort': 'packages desc'}
+        results = get_action('group_list')(context, data_dict)
+        assert results[0]['name'] == u'beta', results[0]['name']
+        assert results[1]['name'] == u'delta', results[1]['name']
+
+        # Test packages forward
+        data_dict = {'all_fields': True, 'sort': 'packages asc'}
+        results = get_action('group_list')(context, data_dict)
+        assert results[-2]['name'] == u'delta', results[-2]['name']
+        assert results[-1]['name'] == u'beta', results[-1]['name']
+
+        # Default ordering for packages
+        data_dict = {'all_fields': True, 'sort': 'packages'}
+        results = get_action('group_list')(context, data_dict)
+        assert results[0]['name'] == u'beta', results[0]['name']
+        assert results[1]['name'] == u'delta', results[1]['name']
+
+
     def test_mainmenu(self):
         # the home page does a package search so have to skip this test if
         # search is not supported
@@ -78,7 +189,9 @@ class TestGroup(FunctionalTestCase):
         group_title = group.title
         group_packages_count = len(group.active_packages().all())
         group_description = group.description
-        self.check_named_element(res, 'tr', group_title, group_packages_count, group_description)
+        self.check_named_element(res, 'tr', group_title,
+                                 group_packages_count,
+                                 group_description)
         res = res.click(group_title)
         assert groupname in res
 
@@ -92,7 +205,8 @@ class TestGroup(FunctionalTestCase):
         plugins.load(plugin)
         name = u'david'
         offset = url_for(controller='group', action='read', id=name)
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert plugin.calls['read'] == 1, plugin.calls
         plugins.unload(plugin)
 
@@ -111,6 +225,7 @@ class TestGroup(FunctionalTestCase):
         res = self.app.get(offset, extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'Add A Group' in res, res
 
+
 class TestGroupWithSearch(FunctionalTestCase):
 
     @classmethod
@@ -138,16 +253,19 @@ class TestGroupWithSearch(FunctionalTestCase):
             assert 'Administrators' in res, res
             assert 'russianfan' in main_res, main_res
             assert name in res, res
-            no_datasets_found = int(re.search('(\d*) datasets found', main_res).groups()[0])
+            no_datasets_found = int(re.search('(\d*) datasets found',
+                                    main_res).groups()[0])
             assert_equal(no_datasets_found, 2)
             pkg = model.Package.by_name(pkgname)
             res = res.click(pkg.title)
             assert '%s - Datasets' % pkg.title in res
 
+
 class TestEdit(FunctionalTestCase):
 
     @classmethod
     def setup_class(self):
+        setup_test_search_index()
         model.Session.remove()
         CreateTestData.create()
         self.groupname = u'david'
@@ -156,6 +274,7 @@ class TestEdit(FunctionalTestCase):
         model.Session.add(model.Package(name=self.packagename))
         model.repo.commit_and_remove()
 
+
     @classmethod
     def teardown_class(self):
         model.Session.remove()
@@ -173,7 +292,8 @@ class TestEdit(FunctionalTestCase):
         group = model.Group.by_name(self.groupname)
         offset = url_for(controller='group', action='edit', id=self.groupname)
         print offset
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'Edit: %s' % group.title in res, res
 
         form = res.forms['group-edit']
@@ -190,7 +310,8 @@ Ho ho ho
         pkg = model.Package.by_name(self.packagename)
         form['packages__2__name'] = pkg.name
 
-        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = form.submit('save', status=302,
+                          extra_environ={'REMOTE_USER': 'russianfan'})
         # should be read page
         # assert 'Groups - %s' % self.groupname in res, res
 
@@ -205,7 +326,8 @@ Ho ho ho
     def test_3_edit_form_has_new_package(self):
         # check for dataset in autocomplete
         offset = url_for(controller='package', action='autocomplete', q='an')
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'annakarenina' in res, res
         assert not 'newone' in res, res
         model.repo.new_revision()
@@ -219,7 +341,8 @@ Ho ho ho
         model.setup_default_user_roles(pkg, [user])
         model.repo.commit_and_remove()
 
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'annakarenina' in res, res
         assert 'newone' in res
 
@@ -234,10 +357,12 @@ Ho ho ho
 
         # Add same package again
         offset = url_for(controller='group', action='edit', id=group_name)
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         fv = res.forms['group-edit']
         fv['packages__1__name'] = self.packagename
-        res = fv.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = fv.submit('save', status=302,
+                        extra_environ={'REMOTE_USER': 'russianfan'})
         res = res.follow()
         assert group_name in res, res
         model.Session.remove()
@@ -251,28 +376,82 @@ Ho ho ho
         plugin = MockGroupControllerPlugin()
         plugins.load(plugin)
         offset = url_for(controller='group', action='edit', id=self.groupname)
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         form = res.forms['group-edit']
         group = model.Group.by_name(self.groupname)
         form['title'] = "huhuhu"
-        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = form.submit('save', status=302,
+                          extra_environ={'REMOTE_USER': 'russianfan'})
         assert plugin.calls['edit'] == 1, plugin.calls
         plugins.unload(plugin)
 
     def test_edit_image_url(self):
         group = model.Group.by_name(self.groupname)
         offset = url_for(controller='group', action='edit', id=self.groupname)
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
 
         form = res.forms['group-edit']
         image_url = u'http://url.to/image_url'
         form['image_url'] = image_url
-        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = form.submit('save', status=302,
+                          extra_environ={'REMOTE_USER': 'russianfan'})
 
         model.Session.remove()
         group = model.Group.by_name(self.groupname)
         assert group.image_url == image_url, group
 
+    def test_edit_change_name(self):
+        group = model.Group.by_name(self.groupname)
+        offset = url_for(controller='group', action='edit', id=self.groupname)
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
+        assert 'Edit: %s' % group.title in res, res
+
+        def update_group(res, name, with_pkg=True):
+            form = res.forms['group-edit']
+            titlefn = 'title'
+            descfn = 'description'
+            newtitle = 'xxxxxxx'
+            newdesc = '''### Lots of stuff here
+
+    Ho ho ho
+    '''
+            form[titlefn] = newtitle
+            form[descfn] = newdesc
+            form['name'] = name
+            if with_pkg:
+                pkg = model.Package.by_name(self.packagename)
+                form['packages__2__name'] = pkg.name
+
+            res = form.submit('save', status=302,
+                              extra_environ={'REMOTE_USER': 'russianfan'})
+        update_group(res, self.groupname, True)
+        update_group(res, 'newname', False)
+
+        model.Session.remove()
+        group = model.Group.by_name('newname')
+
+        # We have the datasets in the DB, but we should also see that many
+        # on the group read page.
+        assert len(group.active_packages().all()) == 3
+
+        offset = url_for(controller='group', action='read', id='newname')
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
+
+        ds = res.body
+        ds = ds[ds.index('datasets') - 10: ds.index('datasets') + 10]
+        assert '3 datasets found' in res, ds
+
+        # reset the group to how we found it
+        offset = url_for(controller='group', action='edit', id='newname')
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
+
+        update_group(res, self.groupname, True)
+
     def test_edit_non_existent(self):
         name = u'group_does_not_exist'
         offset = url_for(controller='group', action='edit', id=name)
@@ -286,7 +465,8 @@ Ho ho ho
 
         group = model.Group.by_name(group_name)
         offset = url_for(controller='group', action='edit', id=group_name)
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         main_res = self.main_div(res)
         assert 'Edit: %s' % group.title in main_res, main_res
         assert 'value="active" selected' in main_res, main_res
@@ -294,7 +474,8 @@ Ho ho ho
         # delete
         form = res.forms['group-edit']
         form['state'] = 'deleted'
-        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = form.submit('save', status=302,
+                          extra_environ={'REMOTE_USER': 'russianfan'})
 
         group = model.Group.by_name(group_name)
         assert_equal(group.state, 'deleted')
@@ -302,6 +483,7 @@ Ho ho ho
         res = res.follow()
         assert res.request.url.startswith('/user/login'), res.request.url
 
+
 class TestNew(FunctionalTestCase):
     groupname = u'david'
 
@@ -336,21 +518,24 @@ class TestNew(FunctionalTestCase):
 
         # Open 'Add A Group' page
         offset = url_for(controller='group', action='new')
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'Add A Group' in res, res
         fv = res.forms['group-edit']
-        assert fv[prefix+'name'].value == '', fv.fields
-        assert fv[prefix+'title'].value == ''
-        assert fv[prefix+'description'].value == ''
-        assert fv['packages__0__name'].value == '', fv['Member--package_name'].value
+        assert fv[prefix + 'name'].value == '', fv.fields
+        assert fv[prefix + 'title'].value == ''
+        assert fv[prefix + 'description'].value == ''
+        assert fv['packages__0__name'].value == '', \
+            fv['Member--package_name'].value
 
         # Edit form
-        fv[prefix+'name'] = group_name
-        fv[prefix+'title'] = group_title
-        fv[prefix+'description'] = group_description
+        fv[prefix + 'name'] = group_name
+        fv[prefix + 'title'] = group_title
+        fv[prefix + 'description'] = group_description
         pkg = model.Package.by_name(self.packagename)
         fv['packages__0__name'] = pkg.name
-        res = fv.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = fv.submit('save', status=302,
+                        extra_environ={'REMOTE_USER': 'russianfan'})
         res = res.follow()
         assert '%s' % group_title in res, res
 
@@ -368,12 +553,14 @@ class TestNew(FunctionalTestCase):
         # Create group
         group_name = u'testgrp1'
         offset = url_for(controller='group', action='new')
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'Add A Group' in res, res
         fv = res.forms['group-edit']
-        assert fv[prefix+'name'].value == '', fv.fields
-        fv[prefix+'name'] = group_name
-        res = fv.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        assert fv[prefix + 'name'].value == '', fv.fields
+        fv[prefix + 'name'] = group_name
+        res = fv.submit('save', status=302,
+                        extra_environ={'REMOTE_USER': 'russianfan'})
         res = res.follow()
         assert group_name in res, res
         model.Session.remove()
@@ -381,12 +568,14 @@ class TestNew(FunctionalTestCase):
         # Create duplicate group
         group_name = u'testgrp1'
         offset = url_for(controller='group', action='new')
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'Add A Group' in res, res
         fv = res.forms['group-edit']
-        assert fv[prefix+'name'].value == '', fv.fields
-        fv[prefix+'name'] = group_name
-        res = fv.submit('save', status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        assert fv[prefix + 'name'].value == '', fv.fields
+        fv[prefix + 'name'] = group_name
+        res = fv.submit('save', status=200,
+                        extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'Group name already exists' in res, res
         self.check_tag(res, '<form', 'has-errors')
         assert 'class="field_error"' in res, res
@@ -395,21 +584,25 @@ class TestNew(FunctionalTestCase):
         plugin = MockGroupControllerPlugin()
         plugins.load(plugin)
         offset = url_for(controller='group', action='new')
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         form = res.forms['group-edit']
         form['name'] = "hahaha"
         form['title'] = "huhuhu"
-        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = form.submit('save', status=302,
+                          extra_environ={'REMOTE_USER': 'russianfan'})
         assert plugin.calls['create'] == 1, plugin.calls
         plugins.unload(plugin)
 
     def test_new_bad_param(self):
-        offset = url_for(controller='group', action='new', __bad_parameter='value')
-        res = self.app.post(offset, {'save':'1'},
+        offset = url_for(controller='group', action='new',
+                         __bad_parameter='value')
+        res = self.app.post(offset, {'save': '1'},
                             extra_environ={'REMOTE_USER': 'russianfan'},
                             status=400)
         assert 'Integrity Error' in res.body
 
+
 class TestRevisions(FunctionalTestCase):
     @classmethod
     def setup_class(self):
@@ -418,7 +611,9 @@ class TestRevisions(FunctionalTestCase):
         self.name = u'revisiontest1'
 
         # create pkg
-        self.description = [u'Written by Puccini', u'Written by Rossini', u'Not written at all', u'Written again', u'Written off']
+        self.description = [u'Written by Puccini', u'Written by Rossini',
+                            u'Not written at all', u'Written again',
+                            u'Written off']
         rev = model.repo.new_revision()
         self.grp = model.Group(name=self.name)
         self.grp.description = self.description[0]
@@ -441,32 +636,42 @@ class TestRevisions(FunctionalTestCase):
         model.repo.rebuild_db()
 
     def test_0_read_history(self):
-        offset = url_for(controller='group', action='history', id=self.grp.name)
+        offset = url_for(controller='group', action='history',
+                         id=self.grp.name)
         res = self.app.get(offset)
         main_res = self.main_div(res)
         assert self.grp.name in main_res, main_res
         assert 'radio' in main_res, main_res
         latest_rev = self.grp.all_revisions[0]
         oldest_rev = self.grp.all_revisions[-1]
-        first_radio_checked_html = '<input checked="checked" id="selected1_%s"' % latest_rev.revision_id
-        assert first_radio_checked_html in main_res, '%s %s' % (first_radio_checked_html, main_res)
-        last_radio_checked_html = '<input checked="checked" id="selected2_%s"' % oldest_rev.revision_id
-        assert last_radio_checked_html in main_res, '%s %s' % (last_radio_checked_html, main_res)
+        first_radio_checked_html = \
+            '<input checked="checked" id="selected1_%s"' % \
+            latest_rev.revision_id
+        assert first_radio_checked_html in main_res, '%s %s' % \
+            (first_radio_checked_html, main_res)
+        last_radio_checked_html = \
+            '<input checked="checked" id="selected2_%s"' % \
+            oldest_rev.revision_id
+        assert last_radio_checked_html in main_res, '%s %s' % \
+            (last_radio_checked_html, main_res)
 
     def test_1_do_diff(self):
-        offset = url_for(controller='group', action='history', id=self.grp.name)
+        offset = url_for(controller='group', action='history',
+                         id=self.grp.name)
         res = self.app.get(offset)
         form = res.forms['group-revisions']
         res = form.submit()
         res = res.follow()
         main_res = self.main_div(res)
-        assert 'error' not in main_res.lower(), main_res
+        assert 'form-errors' not in main_res.lower(), main_res
         assert 'Revision Differences' in main_res, main_res
         assert self.grp.name in main_res, main_res
-        assert '<tr><td>description</td><td><pre>- Written by Puccini\n+ Written off</pre></td></tr>' in main_res, main_res
+        assert "<tr><td>description</td><td><pre>- Written by Puccini\n+" + \
+               " Written off</pre></td></tr>" in main_res, main_res
 
     def test_2_atom_feed(self):
-        offset = url_for(controller='group', action='history', id=self.grp.name)
+        offset = url_for(controller='group', action='history',
+                         id=self.grp.name)
         offset = "%s?format=atom" % offset
         res = self.app.get(offset)
         assert '<feed' in res, res
@@ -499,7 +704,9 @@ class TestOrganizationGroup(FunctionalTestCase):
         group_title = group.title
         group_packages_count = len(group.active_packages().all())
         group_description = group.description
-        self.check_named_element(res, 'tr', group_title, group_packages_count, group_description)
+        self.check_named_element(res, 'tr', group_title,
+                                 group_packages_count,
+                                 group_description)
         res = res.click(group_title)
         assert groupname in res
         assert 'organization' == group.type, group.type
@@ -537,7 +744,8 @@ class TestOrganizationGroup(FunctionalTestCase):
         title = u'Dave\'s books'
         pkgname = u'warandpeace'
         offset = url_for(controller='group', action='edit', id=name)
-        res = self.app.get(offset, extra_environ={'REMOTE_USER': 'russianfan'}, status=200)
+        res = self.app.get(offset,  status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
 
 
 class TestPublisherEdit(FunctionalTestCase):
@@ -561,7 +769,6 @@ class TestPublisherEdit(FunctionalTestCase):
         model.repo.rebuild_db()
         model.Session.remove()
 
-
     def test_0_not_authz(self):
         from pylons import config
         from nose.exc import SkipTest
@@ -584,7 +791,8 @@ class TestPublisherEdit(FunctionalTestCase):
         offset = url_for(controller='group', action='edit', id=self.groupname)
         user = model.User.get('russianfan')
 
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'Edit: %s' % group.title in res, res
 
         form = res.forms['group-edit']
@@ -601,8 +809,8 @@ Ho ho ho
         pkg = model.Package.by_name(self.packagename)
         form['packages__2__name'] = pkg.name
 
-
-        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = form.submit('save', status=302,
+                          extra_environ={'REMOTE_USER': 'russianfan'})
         # should be read page
         # assert 'Groups - %s' % self.groupname in res, res
 
@@ -617,7 +825,8 @@ Ho ho ho
     def test_3_edit_form_has_new_package(self):
         # check for dataset in autocomplete
         offset = url_for(controller='package', action='autocomplete', q='an')
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'annakarenina' in res, res
         assert not 'newone' in res, res
         model.repo.new_revision()
@@ -631,7 +840,8 @@ Ho ho ho
         model.setup_default_user_roles(pkg, [user])
         model.repo.commit_and_remove()
 
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'annakarenina' in res, res
         assert 'newone' in res
 
@@ -646,10 +856,12 @@ Ho ho ho
 
         # Add same package again
         offset = url_for(controller='group', action='edit', id=group_name)
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         fv = res.forms['group-edit']
         fv['packages__1__name'] = self.packagename
-        res = fv.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = fv.submit('save', status=302,
+                        extra_environ={'REMOTE_USER': 'russianfan'})
         res = res.follow()
         assert group_name in res, res
         model.Session.remove()
@@ -663,40 +875,47 @@ Ho ho ho
         plugin = MockGroupControllerPlugin()
         plugins.load(plugin)
         offset = url_for(controller='group', action='edit', id=self.groupname)
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         form = res.forms['group-edit']
         group = model.Group.by_name(self.groupname)
         form['title'] = "huhuhu"
-        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = form.submit('save', status=302,
+                          extra_environ={'REMOTE_USER': 'russianfan'})
         assert plugin.calls['edit'] == 1, plugin.calls
         plugins.unload(plugin)
 
     def test_edit_non_auth(self):
         offset = url_for(controller='group', action='edit', id=self.groupname)
-        res = self.app.get(offset, status=[302,401], extra_environ={'REMOTE_USER': 'non-existent'})
+        res = self.app.get(offset, status=[302, 401],
+                           extra_environ={'REMOTE_USER': 'non-existent'})
 
     def test_edit_fail_auth(self):
-        context = { 'group': model.Group.by_name(self.groupname), 'model': model, 'user': 'russianfan' }
+        context = {'group': model.Group.by_name(self.groupname),
+                   'model': model, 'user': 'russianfan'}
         try:
-            if self.auth.check_access('group_update',context, {}):
-                assert False, "Check access said we were allowed but we shouldn't really"
+            if self.auth.check_access('group_update', context, {}):
+                assert False, "Check access said incorrectly said allowed"
         except NotAuthorized, e:
-            pass # Do nothing as this is what we expected
+            pass  # Do nothing as this is what we expected
 
     def test_edit_success_auth(self):
         userobj = model.User.get('russianfan')
         grp = model.Group.by_name(self.groupname)
 
+        # Monkey patch
+        old_method = model.User.get_groups
         def gg(*args, **kwargs):
             return [grp]
         model.User.get_groups = gg
-
-        context = { 'group': grp, 'model': model, 'user': 'russianfan' }
         try:
-            self.auth.check_access('group_update',context, {})
-        except NotAuthorized, e:
-            assert False, "The user should have access"
-
+            context = { 'group': grp, 'model': model, 'user': 'russianfan' }
+            try:
+                self.auth.check_access('group_update',context, {})
+            except NotAuthorized, e:
+                assert False, "The user should have access"
+        finally:
+            model.User.get_groups = old_method
 
     def test_delete(self):
         group_name = 'deletetest'
@@ -706,7 +925,8 @@ Ho ho ho
 
         group = model.Group.by_name(group_name)
         offset = url_for(controller='group', action='edit', id=group_name)
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = self.app.get(offset, status=200,
+                           extra_environ={'REMOTE_USER': 'russianfan'})
         main_res = self.main_div(res)
         assert 'Edit: %s' % group.title in main_res, main_res
         assert 'value="active" selected' in main_res, main_res
@@ -714,11 +934,11 @@ Ho ho ho
         # delete
         form = res.forms['group-edit']
         form['state'] = 'deleted'
-        res = form.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = form.submit('save', status=302,
+                          extra_environ={'REMOTE_USER': 'russianfan'})
 
         group = model.Group.by_name(group_name)
         assert_equal(group.state, 'deleted')
         res = self.app.get(offset, status=302)
         res = res.follow()
         assert res.request.url.startswith('/user/login'), res.request.url
-
