@@ -13,6 +13,7 @@ from ckan.lib.dictization.model_dictize import (package_dictize,
                                                 activity_dictize,
                                                 package_to_api1,
                                                 package_to_api2,
+                                                user_dictize,
                                                )
 from ckan.lib.dictization.model_save import (package_dict_save,
                                              resource_dict_save,
@@ -175,7 +176,7 @@ class TestBasicDictize:
 
         ## resource
 
-        resource = pkg.resource_groups[0].resources[0]
+        resource = pkg.resource_groups_all[0].resources_all[0]
 
         result = resource_dictize(resource, context)
         self.remove_changable_columns(result)
@@ -364,7 +365,7 @@ class TestBasicDictize:
         anna_dictized = package_dictize(anna1, context)
 
         anna_dictized["name"] = u'annakarenina_changed'
-        anna_dictized["resources"][0]["url"] = u'new_url'
+        anna_dictized["resources"][0]["url"] = u'http://new_url'
 
         model.repo.new_revision()
         package_dict_save(anna_dictized, context)
@@ -375,7 +376,7 @@ class TestBasicDictize:
 
         package_dictized = package_dictize(pkg, context)
 
-        resources_revisions = model.Session.query(model.ResourceRevision).filter_by(resource_group_id=anna1.resource_groups[0].id).all()
+        resources_revisions = model.Session.query(model.ResourceRevision).filter_by(resource_group_id=anna1.resource_groups_all[0].id).all()
 
         sorted_resources = sorted(resources_revisions, key=lambda x: (x.revision_timestamp, x.url))[::-1]
         for res in sorted_resources:
@@ -402,7 +403,7 @@ class TestBasicDictize:
         anna_dictized = package_dictize(anna1, context)
 
         anna_dictized['name'] = u'annakarenina_changed2'
-        anna_dictized['resources'][0]['url'] = u'new_url2'
+        anna_dictized['resources'][0]['url'] = u'http://new_url2'
         anna_dictized['tags'][0]['name'] = u'new_tag'
         anna_dictized['tags'][0].pop('id') #test if
         anna_dictized['extras'][0]['value'] = u'"new_value"'
@@ -426,7 +427,7 @@ class TestBasicDictize:
         assert str(sorted_packages[1].expired_timestamp) != '9999-12-31 00:00:00'
         assert str(sorted_packages[2].expired_timestamp) != '9999-12-31 00:00:00'
 
-        resources_revisions = model.Session.query(model.ResourceRevision).filter_by(resource_group_id=anna1.resource_groups[0].id).all()
+        resources_revisions = model.Session.query(model.ResourceRevision).filter_by(resource_group_id=anna1.resource_groups_all[0].id).all()
         sorted_resources = sorted(resources_revisions, key=lambda x: (x.revision_timestamp, x.url))[::-1]
 
         for pkg in sorted_resources:
@@ -496,7 +497,7 @@ class TestBasicDictize:
         anna_dictized['notes'] = 'wee'
         anna_dictized['resources'].append({
                             'format': u'plain text',
-                            'url': u'newurl'}
+                            'url': u'http://newurl'}
                             )
         anna_dictized['tags'].append({'name': u'newnew_tag'})
         anna_dictized['extras'].append({'key': 'david',
@@ -507,7 +508,7 @@ class TestBasicDictize:
         model.Session.commit()
         model.Session.remove()
 
-        resources_revisions = model.Session.query(model.ResourceRevision).filter_by(resource_group_id=anna1.resource_groups[0].id).all()
+        resources_revisions = model.Session.query(model.ResourceRevision).filter_by(resource_group_id=anna1.resource_groups_all[0].id).all()
 
         sorted_resources = sorted(resources_revisions, key=lambda x: (x.revision_timestamp, x.url))[::-1]
         pprint(anna_dictized['resources'])
@@ -593,7 +594,7 @@ class TestBasicDictize:
         assert sorted_packages[2].state == 'active'
         assert sorted_packages[3].state == 'active'
 
-        resources_revisions = model.Session.query(model.ResourceRevision).filter_by(resource_group_id=anna1.resource_groups[0].id).all()
+        resources_revisions = model.Session.query(model.ResourceRevision).filter_by(resource_group_id=anna1.resource_groups_all[0].id).all()
         sorted_resources = sorted(resources_revisions, key=lambda x: (x.revision_timestamp, x.url))[::-1]
 
         assert len(sorted_resources) == 5
@@ -676,7 +677,7 @@ class TestBasicDictize:
         second_dictized = self.remove_changable_columns(package_dictize(anna1, context))
 
         first_dictized["name"] = u'annakarenina_changed'
-        first_dictized["resources"][0]["url"] = u'new_url'
+        first_dictized["resources"][0]["url"] = u'http://new_url'
 
         assert second_dictized == first_dictized
 
@@ -684,7 +685,7 @@ class TestBasicDictize:
         third_dictized = self.remove_changable_columns(package_dictize(anna1, context))
 
         second_dictized['name'] = u'annakarenina_changed2'
-        second_dictized['resources'][0]['url'] = u'new_url2'
+        second_dictized['resources'][0]['url'] = u'http://new_url2'
         second_dictized['tags'][0]['name'] = u'new_tag'
         second_dictized['tags'][0]['display_name'] = u'new_tag'
         second_dictized['extras'][0]['value'] = u'"new_value"'
@@ -711,7 +712,7 @@ class TestBasicDictize:
             u'size': None,
             u'state': u'active',
             u'tracking_summary': {'total': 0, 'recent': 0},
-            u'url': u'newurl',
+            u'url': u'http://newurl',
             u'webstore_last_updated': None,
             u'webstore_url': None})
 
@@ -741,7 +742,7 @@ class TestBasicDictize:
             'description': u'Full text. Needs escaping: " Umlaut: \xfc',
             'format': u'plain text',
             'tracking_summary': {'recent': 0, 'total': 0},
-            'url': u'test_new',
+            'url': u'http://test_new',
             'cache_url': None,
             'webstore_url': None,
             'cache_last_updated': None,
@@ -760,7 +761,7 @@ class TestBasicDictize:
         model.Session.commit()
         model.Session.remove()
 
-        res = model.Session.query(model.Resource).filter_by(url=u'test_new').one()
+        res = model.Session.query(model.Resource).filter_by(url=u'http://test_new').one()
 
         res_dictized = self.remove_changable_columns(resource_dictize(res, context))
 
@@ -871,14 +872,15 @@ class TestBasicDictize:
         group = model.Session.query(model.Group).filter_by(name=u'help').one()
 
         context = {"model": model,
-                  "session": model.Session}
+                  "session": model.Session,
+                  "user": None}
 
         group_dictized = group_dictize(group, context)
 
         expected = {'description': u'',
                     'extras': [{'key': u'genre', 'state': u'active', 'value': u'"horror"'},
                                {'key': u'media', 'state': u'active', 'value': u'"dvd"'}],
-                    'tags': [{'capacity': 'public', 'name': u'russian'}],
+                    'tags': [{'capacity': u'public', 'display_name': u'russian', 'name': u'russian'}],
                     'groups': [{'description': u'',
                                'capacity' : 'public',
                                'display_name': u'simple',
@@ -892,13 +894,11 @@ class TestBasicDictize:
                     'users': [{'about': u'I love reading Annakarenina. My site: <a href="http://anna.com">anna.com</a>',
                               'display_name': u'annafan',
                               'capacity' : 'public',
-                              'email': None,
                               'email_hash': 'd41d8cd98f00b204e9800998ecf8427e',
                               'fullname': None,
                               'name': u'annafan',
                               'number_administered_packages': 1L,
-                              'number_of_edits': 0L,
-                              'reset_key': None}],
+                              'number_of_edits': 0L}],
                     'name': u'help',
                     'display_name': u'help',
                     'image_url': u'',
@@ -1068,3 +1068,98 @@ class TestBasicDictize:
         assert_not_in('test-group-2', [ g['name'] for g in result['groups'] ])
         assert_in('test-group-1', [ g['name'] for g in result['groups'] ])
 
+    def test_22_user_dictize_as_sysadmin(self):
+        '''Sysadmins should be allowed to see certain sensitive data.'''
+        context = {
+            'model': model,
+            'session': model.Session,
+            'user': 'testsysadmin',
+        }
+
+        user = model.User.by_name('tester')
+
+        user_dict = user_dictize(user, context)
+
+        # Check some of the non-sensitive data
+        assert 'name' in user_dict
+        assert 'about' in user_dict
+
+        # Check sensitive data is available
+        assert 'apikey' in user_dict
+        assert 'reset_key' in user_dict
+        assert 'email' in user_dict
+
+        # Passwords should never be available
+        assert 'password' not in user_dict
+
+    def test_23_user_dictize_as_same_user(self):
+        '''User should be able to see their own sensitive data.'''
+        context = {
+            'model': model,
+            'session': model.Session,
+            'user': 'tester',
+        }
+
+        user = model.User.by_name('tester')
+
+        user_dict = user_dictize(user, context)
+
+        # Check some of the non-sensitive data
+        assert 'name' in user_dict
+        assert 'about' in user_dict
+
+        # Check sensitive data is available
+        assert 'apikey' in user_dict
+        assert 'reset_key' in user_dict
+        assert 'email' in user_dict
+
+        # Passwords should never be available
+        assert 'password' not in user_dict
+
+    def test_24_user_dictize_as_other_user(self):
+        '''User should not be able to see other's sensitive data.'''
+        context = {
+            'model': model,
+            'session': model.Session,
+            'user': 'annafan',
+        }
+
+        user = model.User.by_name('tester')
+
+        user_dict = user_dictize(user, context)
+
+        # Check some of the non-sensitive data
+        assert 'name' in user_dict
+        assert 'about' in user_dict
+
+        # Check sensitive data is not available
+        assert 'apikey' not in user_dict
+        assert 'reset_key' not in user_dict
+        assert 'email' not in user_dict
+
+        # Passwords should never be available
+        assert 'password' not in user_dict
+
+    def test_25_user_dictize_as_anonymous(self):
+        '''Anonymous should not be able to see other's sensitive data.'''
+        context = {
+            'model': model,
+            'session': model.Session,
+            'user': '',
+        }
+
+        user = model.User.by_name('tester')
+
+        user_dict = user_dictize(user, context)
+
+        # Check some of the non-sensitive data
+        assert 'name' in user_dict
+        assert 'about' in user_dict
+
+        # Check sensitive data is not available
+        assert 'apikey' not in user_dict
+        assert 'reset_key' not in user_dict
+        assert 'email' not in user_dict
+
+        # Passwords should never be available
+        assert 'password' not in user_dict
