@@ -15,6 +15,7 @@ from ckan.tests import setup_test_search_index, search_related
 from ckan.tests import StatusCodes
 from ckan.logic import get_action, NotAuthorized
 from ckan.logic.action import get_domain_object
+from ckan.tests import TestRoles
 
 from ckan import plugins
 from ckan.plugins import SingletonPlugin, implements, IPackageController
@@ -55,12 +56,13 @@ class TestAction(WsgiAppCase):
 
     def test_01_package_list(self):
         postparams = '%s=1' % json.dumps({})
-        res = self.app.post('/api/action/package_list', params=postparams)
-        assert_dicts_equal_ignoring_ordering(
-            json.loads(res.body),
-            {"help": "Lists packages by name or id",
-             "success": True,
-             "result": ["annakarenina", "warandpeace"]})
+        res = json.loads(self.app.post('/api/action/package_list', params=postparams).body)
+        assert res['success'] is True
+        assert len(res['result']) == 2
+        assert 'warandpeace' in res['result']
+        assert 'annakarenina' in res['result']
+        assert res['help'].startswith(
+            "Return a list of the names of the site's datasets (packages).")
 
     def test_01_package_show(self):
         anna_id = model.Package.by_name(u'annakarenina').id
@@ -68,7 +70,8 @@ class TestAction(WsgiAppCase):
         res = self.app.post('/api/action/package_show', params=postparams)
         res_dict = json.loads(res.body)
         assert_equal(res_dict['success'], True)
-        assert_equal(res_dict['help'], None)
+        assert res_dict['help'].startswith(
+            "Return the metadata of a dataset (package) and its resources.")
         pkg = res_dict['result']
         assert_equal(pkg['name'], 'annakarenina')
         missing_keys = set(('title', 'groups')) - set(pkg.keys())
@@ -84,7 +87,8 @@ class TestAction(WsgiAppCase):
         msg = res.body[len('jsoncallback')+1:-2]
         res_dict = json.loads(msg)
         assert_equal(res_dict['success'], True)
-        assert_equal(res_dict['help'], None)
+        assert res_dict['help'].startswith(
+            "Return the metadata of a dataset (package) and its resources.")
         pkg = res_dict['result']
         assert_equal(pkg['name'], 'annakarenina')
         missing_keys = set(('title', 'groups')) - set(pkg.keys())
@@ -185,11 +189,40 @@ class TestAction(WsgiAppCase):
         res = self.app.post('/api/action/package_create', params=postparams,
                                      status=StatusCodes.STATUS_403_ACCESS_DENIED)
 
+    def test_41_create_resource(self):
+
+        anna_id = model.Package.by_name(u'annakarenina').id
+        resource = {'package_id': anna_id, 'url': 'http://new_url'}
+        api_key = model.User.get('annafan').apikey.encode('utf8')
+        postparams = '%s=1' % json.dumps(resource)
+        res = self.app.post('/api/action/resource_create', params=postparams,
+                            extra_environ={'Authorization': api_key })
+
+        resource = json.loads(res.body)['result']
+
+        assert resource['url'] == 'http://new_url'
+
+    def test_42_create_resource_with_error(self):
+
+        anna_id = model.Package.by_name(u'annakarenina').id
+        resource = {'package_id': anna_id, 'url': 'new_url', 'created': 'bad_date'}
+        api_key = model.User.get('annafan').apikey.encode('utf8')
+
+        postparams = '%s=1' % json.dumps(resource)
+        res = self.app.post('/api/action/resource_create', params=postparams,
+                            extra_environ={'Authorization': api_key},
+                            status=StatusCodes.STATUS_409_CONFLICT)
+
+        assert json.loads(res.body)['error'] ==  {"__type": "Validation Error", "created": ["Date format incorrect"]}
+
+
+
     def test_04_user_list(self):
         postparams = '%s=1' % json.dumps({})
         res = self.app.post('/api/action/user_list', params=postparams)
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Lists the current users'
+        assert res_obj['help'].startswith(
+                "Return a list of the site's user accounts.")
         assert res_obj['success'] == True
         assert len(res_obj['result']) == 7
         assert res_obj['result'][0]['name'] == 'annafan'
@@ -201,7 +234,7 @@ class TestAction(WsgiAppCase):
         postparams = '%s=1' % json.dumps({'id':'annafan'})
         res = self.app.post('/api/action/user_show', params=postparams)
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Shows user details'
+        assert res_obj['help'].startswith("Return a user account.")
         assert res_obj['success'] == True
         result = res_obj['result']
         assert result['name'] == 'annafan'
@@ -238,7 +271,7 @@ class TestAction(WsgiAppCase):
         postparams = '%s=1' % json.dumps({'id':'tester'})
         res = self.app.post('/api/action/user_show', params=postparams)
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Shows user details'
+        assert res_obj['help'].startswith("Return a user account.")
         assert res_obj['success'] == True
         result = res_obj['result']
         assert result['name'] == 'tester'
@@ -274,16 +307,14 @@ class TestAction(WsgiAppCase):
                             extra_environ={'Authorization': str(self.sysadmin_user.apikey)},
                             status=StatusCodes.STATUS_409_CONFLICT)
         res_obj = json.loads(res.body)
-        assert res_obj == {
-            'error': {
+        assert res_obj['error'] == {
                 '__type': 'Validation Error',
                 'name': ['Missing value'],
                 'email': ['Missing value'],
                 'password': ['Missing value']
-            },
-            'help': 'Creates a new user',
-            'success': False
-        }
+            }
+        assert res_obj['help'].startswith("Create a new user.")
+        assert res_obj['success'] is False
 
     def test_11_user_create_wrong_password(self):
         user_dict = {'name':'test_create_from_action_api_2',
@@ -296,14 +327,10 @@ class TestAction(WsgiAppCase):
                             status=StatusCodes.STATUS_409_CONFLICT)
 
         res_obj = json.loads(res.body)
-        assert res_obj == {
-            'error': {
-                '__type': 'Validation Error',
-                'password': ['Your password must be 4 characters or longer']
-            },
-            'help': 'Creates a new user',
-            'success': False
-        }
+        assert res_obj['help'].startswith('Create a new user.')
+        assert res_obj['success'] is False
+        assert res_obj['error'] == { '__type': 'Validation Error',
+                'password': ['Your password must be 4 characters or longer']}
 
     def test_12_user_update(self):
         normal_user_dict = {'id': self.normal_user.id,
@@ -323,7 +350,7 @@ class TestAction(WsgiAppCase):
                             extra_environ={'Authorization': str(self.normal_user.apikey)})
 
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Updates the user\'s details'
+        assert res_obj['help'].startswith("Update a user account.")
         assert res_obj['success'] == True
         result = res_obj['result']
         assert result['id'] == self.normal_user.id
@@ -343,7 +370,7 @@ class TestAction(WsgiAppCase):
                             extra_environ={'Authorization': str(self.sysadmin_user.apikey)})
 
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Updates the user\'s details'
+        assert res_obj['help'].startswith("Update a user account.")
         assert res_obj['success'] == True
         result = res_obj['result']
         assert result['id'] == self.sysadmin_user.id
@@ -357,7 +384,7 @@ class TestAction(WsgiAppCase):
                             extra_environ={'Authorization': str(self.sysadmin_user.apikey)})
 
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Updates the user\'s details'
+        assert res_obj['help'].startswith("Update a user account.")
         assert res_obj['success'] == True
         result = res_obj['result']
         assert result['id'] == self.normal_user.id
@@ -372,14 +399,12 @@ class TestAction(WsgiAppCase):
                             status=StatusCodes.STATUS_403_ACCESS_DENIED)
 
         res_obj = json.loads(res.body)
-        assert res_obj == {
-            'error': {
+        assert res_obj['help'].startswith("Update a user account.")
+        assert res_obj['error'] == {
                 '__type': 'Authorization Error',
                 'message': 'Access denied'
-            },
-            'help': 'Updates the user\'s details',
-            'success': False
-        }
+            }
+        assert res_obj['success'] is False
 
     def test_12_user_update_errors(self):
         test_calls = (
@@ -418,16 +443,10 @@ class TestAction(WsgiAppCase):
         postparams = '%s=1' % json.dumps({})
         res = self.app.post('/api/action/group_list', params=postparams)
         res_obj = json.loads(res.body)
-        assert_dicts_equal_ignoring_ordering(
-            res_obj,
-            {
-                'result': [
-                    'david',
-                    'roger',
-                    ],
-                'help': 'Returns a list of groups',
-                'success': True
-            })
+        assert res_obj['result'] == ['david', 'roger']
+        assert res_obj['success'] is True
+        assert res_obj['help'].startswith(
+                "Return a list of the names of the site's groups.")
 
         #Get all fields
         postparams = '%s=1' % json.dumps({'all_fields':True})
@@ -468,7 +487,7 @@ class TestAction(WsgiAppCase):
         postparams = '%s=1' % json.dumps({'id':'david'})
         res = self.app.post('/api/action/group_show', params=postparams)
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Shows group details'
+        assert res_obj['help'].startswith("Return the details of a group.")
         assert res_obj['success'] == True
         result = res_obj['result']
         assert result['name'] == 'david'
@@ -485,26 +504,22 @@ class TestAction(WsgiAppCase):
 
         res_obj = json.loads(res.body)
         pprint(res_obj)
-        assert res_obj == {
-            'error': {
+        assert res_obj['error'] == {
                 '__type': 'Not Found Error',
                 'message': 'Not found'
-            },
-            'help': 'Shows group details',
-            'success': False
-        }
-
+            }
+        assert res_obj['help'].startswith('Return the details of a group.')
+        assert res_obj['success'] is False
 
     def test_16_user_autocomplete(self):
         #Empty query
         postparams = '%s=1' % json.dumps({})
         res = self.app.post('/api/action/user_autocomplete', params=postparams)
         res_obj = json.loads(res.body)
-        assert res_obj == {
-            'help': 'Returns users containing the provided string',
-            'result': [],
-            'success': True
-        }
+        assert res_obj['help'].startswith(
+                "Return a list of user names that contain a string.")
+        assert res_obj['result'] == []
+        assert res_obj['success'] is True
 
         #Normal query
         postparams = '%s=1' % json.dumps({'q':'joe'})
@@ -639,12 +654,9 @@ class TestAction(WsgiAppCase):
             status=StatusCodes.STATUS_403_ACCESS_DENIED
         )
         res_obj = json.loads(res.body)
-        expected_res_obj = {
-            'help': None,
-            'success': False,
-            'error': {'message': 'Access denied', '__type': 'Authorization Error'}
-        }
-        assert res_obj == expected_res_obj, res_obj
+        assert res_obj['help'].startswith("Update a task status.")
+        assert res_obj['success'] is False
+        assert res_obj['error'] == {'message': 'Access denied', '__type': 'Authorization Error'}
 
     def test_23_task_status_validation(self):
         task_status = {}
@@ -859,17 +871,17 @@ class TestAction(WsgiAppCase):
         annafan = model.User.by_name(u'annafan')
         authgroup = model.AuthorizationGroup.by_name(u'anauthzgroup')
         authgroup2 = model.AuthorizationGroup.by_name(u'anotherauthzgroup')
-        
+
         model.add_authorization_group_to_role(authgroup2, 'editor', authgroup)
         model.repo.commit_and_remove()
-        
+
         postparams = '%s=1' % json.dumps({'domain_object': authgroup.id,
                                           'authorization_group': authgroup2.id})
         res = self.app.post('/api/action/roles_show', params=postparams,
                             extra_environ={'Authorization': str(annafan.apikey)},
                             status=200)
-        
-        authgroup_roles = self.get_roles(authgroup.id, authgroup_ref=authgroup2.name)
+
+        authgroup_roles = TestRoles.get_roles(authgroup.id, authgroup_ref=authgroup2.name)
         assert_equal(authgroup_roles, ['"anotherauthzgroup" is "editor" on "anauthzgroup"'])
 
     def test_35_user_role_update(self):
@@ -893,54 +905,19 @@ class TestAction(WsgiAppCase):
         assert_equal(results['roles'][0]['role'], 'reader')
         assert_equal(results['roles'][0]['package_id'], anna.id)
         assert_equal(results['roles'][0]['user_id'], tester.id)
-        
+
         roles_after = get_action('roles_show') \
                       ({'model': model, 'session': model.Session}, \
                        {'domain_object': anna.id,
                         'user': 'tester'})
         assert_equal(results['roles'], roles_after['roles'])
 
-    def get_roles(self, domain_object_ref, user_ref=None, authgroup_ref=None,
-                  prettify=True):
-        data_dict = {'domain_object': domain_object_ref}
-        if user_ref:
-            data_dict['user'] = user_ref
-        if authgroup_ref:
-            data_dict['authorization_group'] = authgroup_ref
-        role_dicts = get_action('roles_show') \
-                     ({'model': model, 'session': model.Session}, \
-                      data_dict)['roles']
-        if prettify:
-            role_dicts = self.prettify_role_dicts(role_dicts)
-        return role_dicts
-
-    def prettify_role_dicts(self, role_dicts, one_per_line=True):
-        '''Replace ids with names'''
-        pretty_roles = []
-        for role_dict in role_dicts:
-            pretty_role = {}
-            for key, value in role_dict.items():
-                if key.endswith('_id') and value and key != 'user_object_role_id':
-                    pretty_key = key[:key.find('_id')]
-                    domain_object = get_domain_object(model, value)
-                    pretty_value = domain_object.name
-                    pretty_role[pretty_key] = pretty_value
-                else:
-                    pretty_role[key] = value
-            if one_per_line:
-                pretty_role = '"%s" is "%s" on "%s"' % (
-                    pretty_role.get('user') or pretty_role.get('authorized_group'),
-                    pretty_role['role'],
-                    pretty_role.get('package') or pretty_role.get('group') or pretty_role.get('authorization_group'))
-            pretty_roles.append(pretty_role)
-        return pretty_roles
-
     def test_36_user_role_update_for_auth_group(self):
         anna = model.Package.by_name(u'annakarenina')
         annafan = model.User.by_name(u'annafan')
         authgroup = model.AuthorizationGroup.by_name(u'anauthzgroup')
-        all_roles_before = self.get_roles(anna.id)
-        authgroup_roles_before = self.get_roles(anna.id, authgroup_ref=authgroup.name)
+        all_roles_before = TestRoles.get_roles(anna.id)
+        authgroup_roles_before = TestRoles.get_roles(anna.id, authgroup_ref=authgroup.name)
         assert_equal(len(authgroup_roles_before), 0)
         postparams = '%s=1' % json.dumps({'authorization_group': authgroup.name,
                                           'domain_object': anna.id,
@@ -958,12 +935,12 @@ class TestAction(WsgiAppCase):
         assert_equal(results['roles'][0]['role'], 'editor')
         assert_equal(results['roles'][0]['package_id'], anna.id)
         assert_equal(results['roles'][0]['authorized_group_id'], authgroup.id)
-        
-        all_roles_after = self.get_roles(anna.id)
-        authgroup_roles_after = self.get_roles(anna.id, authgroup_ref=authgroup.name)
+
+        all_roles_after = TestRoles.get_roles(anna.id)
+        authgroup_roles_after = TestRoles.get_roles(anna.id, authgroup_ref=authgroup.name)
         assert_equal(set(all_roles_before) ^ set(all_roles_after),
                      set([u'"anauthzgroup" is "editor" on "annakarenina"']))
-        
+
         roles_after = get_action('roles_show') \
                       ({'model': model, 'session': model.Session}, \
                        {'domain_object': anna.id,
@@ -983,8 +960,8 @@ class TestAction(WsgiAppCase):
     def test_38_user_role_bulk_update(self):
         anna = model.Package.by_name(u'annakarenina')
         annafan = model.User.by_name(u'annafan')
-        all_roles_before = self.get_roles(anna.id)
-        user_roles_before = self.get_roles(anna.id, user_ref=annafan.name)
+        all_roles_before = TestRoles.get_roles(anna.id)
+        user_roles_before = TestRoles.get_roles(anna.id, user_ref=annafan.name)
         roles_before = get_action('roles_show') \
                                  ({'model': model, 'session': model.Session}, \
                                   {'domain_object': anna.id})
@@ -1002,8 +979,8 @@ class TestAction(WsgiAppCase):
         results = json.loads(res.body)['result']
 
         # check there are 2 new roles (not 3 because annafan is already admin)
-        all_roles_after = self.get_roles(anna.id)
-        user_roles_after = self.get_roles(anna.id, user_ref=annafan.name)
+        all_roles_after = TestRoles.get_roles(anna.id)
+        user_roles_after = TestRoles.get_roles(anna.id, user_ref=annafan.name)
         assert_equal(set(all_roles_before) ^ set(all_roles_after),
                      set([u'"annafan" is "editor" on "annakarenina"',
                           u'"russianfan" is "editor" on "annakarenina"']))
@@ -1031,12 +1008,107 @@ class TestAction(WsgiAppCase):
                INSERT INTO celery_taskmeta (id, task_id, status, result, date_done, traceback) VALUES (2, '51f2105d-85b1-4393-b821-ac11475919d9', 'FAILURE', '52e', '2012-04-20 21:33:01.622557', 'Traceback')'''
         )
         model.Session.commit()
-        res = self.app.post('/api/action/resource_status_show', 
+        res = json.loads(self.app.post('/api/action/resource_status_show',
                             params=json.dumps({'id': '749cdcf2-3fc8-44ae-aed0-5eff8cc5032c'}),
-                            status=200)
+                            status=200).body)
 
-        assert json.loads(res.body) == {"help": None, "success": True, "result": [{"status": "FAILURE", "entity_id": "749cdcf2-3fc8-44ae-aed0-5eff8cc5032c", "task_type": "qa", "last_updated": "2012-04-20T21:32:45.553986", "date_done": "2012-04-20T21:33:01.622557", "entity_type": "resource", "traceback": "Traceback", "value": "51f2105d-85b1-4393-b821-ac11475919d9", "state": None, "key": "celery_task_id", "error": "", "id": "5753adae-cd0d-4327-915d-edd832d1c9a3"}]}
+        assert res['help'].startswith(
+                "Return the statuses of a resource's tasks.")
+        assert res['success'] is True
+        assert res['result'] == [{"status": "FAILURE", "entity_id": "749cdcf2-3fc8-44ae-aed0-5eff8cc5032c", "task_type": "qa", "last_updated": "2012-04-20T21:32:45.553986", "date_done": "2012-04-20T21:33:01.622557", "entity_type": "resource", "traceback": "Traceback", "value": "51f2105d-85b1-4393-b821-ac11475919d9", "state": None, "key": "celery_task_id", "error": "", "id": "5753adae-cd0d-4327-915d-edd832d1c9a3"}]
 
+    def test_41_missing_action(self):
+        try:
+            get_action('unicorns')
+            assert False, "We found a non-existent action"
+        except KeyError:
+            assert True
+
+    def test_42_resource_search_with_single_field_query(self):
+        request_body = {
+            'query': ["description:index"],
+        }
+        postparams = json.dumps(request_body)
+        response = self.app.post('/api/action/resource_search',
+                                 params=postparams)
+        result = json.loads(response.body)['result']['results']
+        count = json.loads(response.body)['result']['count']
+
+        ## Due to the side-effect of previously run tests, there may be extra
+        ## resources in the results.  So just check that each found Resource
+        ## matches the search criteria
+        assert count > 0
+        for resource in result:
+            assert "index" in resource['description'].lower()
+
+    def test_42_resource_search_across_multiple_fields(self):
+        request_body = {
+            'query': ["description:index", "format:json"],
+        }
+        postparams = json.dumps(request_body)
+        response = self.app.post('/api/action/resource_search',
+                                 params=postparams)
+        result = json.loads(response.body)['result']['results']
+        count = json.loads(response.body)['result']['count']
+
+        ## Due to the side-effect of previously run tests, there may be extra
+        ## resources in the results.  So just check that each found Resource
+        ## matches the search criteria
+        assert count > 0
+        for resource in result:
+            assert "index" in resource['description'].lower()
+            assert "json" in resource['format'].lower()
+
+    def test_42_resource_search_test_percentage_is_escaped(self):
+        request_body = {
+            'query': ["description:index%"],
+        }
+        postparams = json.dumps(request_body)
+        response = self.app.post('/api/action/resource_search',
+                                 params=postparams)
+        count = json.loads(response.body)['result']['count']
+
+        # There shouldn't be any results.  If the '%' character wasn't
+        # escaped correctly, then the search would match because of the
+        # unescaped wildcard.
+        assert count is 0
+
+    def test_42_resource_search_fields_parameter_still_accepted(self):
+        '''The fields parameter is deprecated, but check it still works.
+
+        Remove this test when removing the fields parameter.  (#2603)
+        '''
+        request_body = {
+            'fields': {"description": "index"},
+        }
+
+        postparams = json.dumps(request_body)
+        response = self.app.post('/api/action/resource_search',
+                                 params=postparams)
+        result = json.loads(response.body)['result']['results']
+        count = json.loads(response.body)['result']['count']
+
+        ## Due to the side-effect of previously run tests, there may be extra
+        ## resources in the results.  So just check that each found Resource
+        ## matches the search criteria
+        assert count > 0
+        for resource in result:
+            assert "index" in resource['description'].lower()
+
+    def test_42_resource_search_accessible_via_get_request(self):
+        response = self.app.get('/api/action/resource_search'
+                                '?query=description:index&query=format:json')
+
+        result = json.loads(response.body)['result']['results']
+        count = json.loads(response.body)['result']['count']
+
+        ## Due to the side-effect of previously run tests, there may be extra
+        ## resources in the results.  So just check that each found Resource
+        ## matches the search criteria
+        assert count > 0
+        for resource in result:
+            assert "index" in resource['description'].lower()
+            assert "json" in resource['format'].lower()
 
 class TestActionTermTranslation(WsgiAppCase):
 
@@ -1247,7 +1319,7 @@ class MockPackageSearchPlugin(SingletonPlugin):
 
         assert 'results' in search_results
         assert 'count' in search_results
-        assert 'facets' in search_results
+        assert 'search_facets' in search_results
 
         if 'extras' in search_params and 'ext_avoid' in search_params['extras']:
             # Remove results with a certain value
@@ -1261,7 +1333,7 @@ class MockPackageSearchPlugin(SingletonPlugin):
         return search_results
 
     def before_view(self, data_dict):
-        
+
         data_dict['title'] = 'string_not_found_in_rest_of_template'
 
         return data_dict
@@ -1327,7 +1399,7 @@ class TestSearchPluginInterface(WsgiAppCase):
         res = self.app.post('/api/action/package_search', params=search_params)
 
         res_dict = json.loads(res.body)['result']
-        assert res_dict['count'] == 0 
+        assert res_dict['count'] == 0
         assert len(res_dict['results']) == 0
 
         # all datasets should get abcabcabc
@@ -1344,8 +1416,8 @@ class TestSearchPluginInterface(WsgiAppCase):
         res = self.app.get('/dataset/annakarenina')
 
         assert 'string_not_found_in_rest_of_template' in res.body
-        
+
         res = self.app.get('/dataset?q=')
         assert res.body.count('string_not_found_in_rest_of_template') == 2
-        
+
 
