@@ -28,7 +28,7 @@ class TestAction(WsgiAppCase):
                 u'Flexible \u30a1', 'tollbooth', 'tolkien', 'toledo',
                 'tolerance'])
         assert resbody['help'].startswith(
-                'Return a list of tag dictionaries.')
+                "Return a list of the site's tags.")
         #Get all fields
         postparams = '%s=1' % json.dumps({'all_fields':True})
         res = self.app.post('/api/action/tag_list', params=postparams)
@@ -44,18 +44,6 @@ class TestAction(WsgiAppCase):
         assert res_obj['result'][russian_index]['name'] == 'russian'
         assert res_obj['result'][tolstoy_index]['name'] == 'tolstoy'
 
-        # The "moo" package may part of the retrieved packages, depending
-        # upon whether this test is run in isolation from the rest of the
-        # test suite or not.
-        number_of_russian_packages = len(res_obj['result'][russian_index]['packages'])   # warandpeace, annakarenina (moo?)
-        number_of_tolstoy_packages = len(res_obj['result'][tolstoy_index]['packages'])   # annakarenina
-        number_of_flexible_packages = len(res_obj['result'][flexible_index]['packages']) # warandpeace, annakarenina (moo?)
-
-        # Assert we have the correct number of packages, independantly of
-        # whether the "moo" package may exist or not.
-        assert number_of_russian_packages - number_of_tolstoy_packages == 1
-        assert number_of_flexible_packages == (number_of_russian_packages - number_of_tolstoy_packages) + 1
-
         assert 'id' in res_obj['result'][0]
         assert 'id' in res_obj['result'][1]
         assert 'id' in res_obj['result'][2]
@@ -85,7 +73,7 @@ class TestAction(WsgiAppCase):
         body = json.loads(res.body)
         assert body['success'] is True
         assert body['result'] == [tag_name]
-        assert body['help'].startswith('Return a list of tag dictionaries.')
+        assert body['help'].startswith("Return a list of the site's tags.")
 
         # check that invalid vocab name results in a 404
         params = '%s=1' % json.dumps({'vocabulary_id': 'invalid-vocab-name'})
@@ -95,7 +83,8 @@ class TestAction(WsgiAppCase):
         postparams = '%s=1' % json.dumps({'id':'russian'})
         res = self.app.post('/api/action/tag_show', params=postparams)
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Shows tag details'
+        assert res_obj['help'].startswith(
+                "Return the details of a tag and all its datasets.")
         assert res_obj['success'] == True
         result = res_obj['result']
         assert result['name'] == 'russian'
@@ -122,7 +111,8 @@ class TestAction(WsgiAppCase):
         postparams = '%s=1' % json.dumps({'id':u'Flexible \u30a1'})
         res = self.app.post('/api/action/tag_show', params=postparams)
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Shows tag details'
+        assert res_obj['help'].startswith(
+                "Return the details of a tag and all its datasets.")
         assert res_obj['success'] == True
         result = res_obj['result']
         assert result['name'] == u'Flexible \u30a1'
@@ -157,9 +147,9 @@ class TestAction(WsgiAppCase):
         res = self.app.post('/api/action/user_create', params=postparams,
                             status=StatusCodes.STATUS_403_ACCESS_DENIED)
         res_obj = json.loads(res.body)
-        assert res_obj == {'help': 'Creates a new user',
-                           'success': False,
-                           'error': {'message': 'Access denied', '__type': 'Authorization Error'}}
+        assert res_obj['help'].startswith("Create a new user.")
+        assert res_obj['success'] is False
+        assert res_obj['error'] == {'message': 'Access denied', '__type': 'Authorization Error'}
 
     def test_09_user_create(self):
         user_dict = {'name':'test_create_from_action_api',
@@ -171,7 +161,7 @@ class TestAction(WsgiAppCase):
         res = self.app.post('/api/action/user_create', params=postparams,
                             extra_environ={'Authorization': str(self.sysadmin_user.apikey)})
         res_obj = json.loads(res.body)
-        assert res_obj['help'] == 'Creates a new user'
+        assert res_obj['help'].startswith("Create a new user.")
         assert res_obj['success'] == True
         result = res_obj['result']
         assert result['name'] == user_dict['name']
@@ -212,6 +202,16 @@ class TestAction(WsgiAppCase):
         assert len(tag_dicts) == 1
         assert tag_dicts[0]['name'] == 'russian'
 
+    def test_15a_tag_search_with_one_match_using_fields_parameter(self):
+        paramd = {'fields': {'tags': 'russ'} }
+        params = json.dumps(paramd)
+        res = self.app.post('/api/action/tag_search', params=params)
+        assert res.json['success'] is True
+        assert res.json['result']['count'] == 1
+        tag_dicts = res.json['result']['results']
+        assert len(tag_dicts) == 1
+        assert tag_dicts[0]['name'] == 'russian'
+
     def test_15a_tag_search_with_many_matches(self):
         paramd = {'q': 'tol' }
         params = json.dumps(paramd)
@@ -291,7 +291,7 @@ class TestAction(WsgiAppCase):
         assert res_obj['success'] == True
         assert res_obj['result'] == []
         assert res_obj['help'].startswith(
-                'Return a list of tag names that contain the given string.')
+                "Return a list of tag names that contain a given string.")
 
         #Normal query
         postparams = '%s=1' % json.dumps({'q':'r'})
@@ -300,7 +300,7 @@ class TestAction(WsgiAppCase):
         assert res_obj['success'] == True
         assert res_obj['result'] == ['russian', 'tolerance']
         assert res_obj['help'].startswith(
-                'Return a list of tag names that contain the given string.')
+                'Return a list of tag names that contain a given string.')
 
     def test_15_tag_autocomplete_tag_with_spaces(self):
         """Asserts autocomplete finds tags that contain spaces"""
